import glob
import os

import tqdm
import sys
sys.path.insert(0, "../../../")
from urllib.parse import unquote
from gxl_ai_utils.utils import utils_spider, utils_file

SAVE_DIR = '/home/work_nfs14/xlgeng/tts_data_pachong_high_quality/voicewiki_gxl/'
# SAVE_DIR = './output/voicewiki_gxl/'
utils_file.makedir(SAVE_DIR)

class Gxl_Spider_VoiceWiki:
    def __init__(self):
        super().__init__()
        self.url = 'https://voicewiki.cn'

    def get_url_from_one_page(self, input_url, url_list_file='./output/voicewike/police_audio_url.jsonl'):
        xpath4num = '//*[@id="mw-content-text"]/div/table'
        page_txt = utils_spider.send_request(input_url).text
        audio_num = len(utils_spider.handle_xpath(page_txt, xpath4num))
        utils_file.logging_print("该页面存在的音频数量为:" + str(audio_num))
        res_list = []
        for i in range(2, audio_num + 1):
            en_link_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/th/div/div[2]/div/div[2]/a/@href'
            cn_link_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[2]/th/div/div[2]/div/div[2]/a/@href'
            en_text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/td//text()'
            cn_text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[2]/td//text()'
            en_link = utils_spider.handle_xpath(page_txt, en_link_xpath)
            cn_link = utils_spider.handle_xpath(page_txt, cn_link_xpath)
            en_text = utils_spider.handle_xpath(page_txt, en_text_xpath)
            cn_text = utils_spider.handle_xpath(page_txt, cn_text_xpath)
            temp_dict = {}
            if len(en_link) != 0:
                temp_dict["en_link"] = self.url + en_link[0]
            else:
                temp_dict["en_link"] = ""
            if len(cn_link) != 0:
                temp_dict["cn_link"] = self.url + cn_link[0]
            else:
                temp_dict["cn_link"] = ""
            if len(en_text) != 0:
                temp_dict["en_text"] = en_text[0].strip()
            else:
                temp_dict["en_text"] = ""
            if len(cn_text) != 0:
                temp_dict["cn_text"] = cn_text[0].strip()
            else:
                temp_dict["cn_text"] = ""
            if len(en_link) != 0 or len(cn_link) != 0 or len(en_text) != 0 or len(cn_text) != 0:
                res_list.append(temp_dict)
                utils_file.logging_print("耿雪龙:temp_dict不为空,",temp_dict)
            else:
                utils_file.logging_print("warning:temp_dict为空, i:", i)
        utils_file.write_dict_list_to_jsonl(res_list, url_list_file)

    def start_download_from_one_page_url_file(self, url_list_file='./output/voicewike/police_audio_url.jsonl',
                                              download_dir='./output/voicewike/police_audio',
                                              output_list_file='./output/voicewike/police_audio_path.jsonl'):
        res_list = []
        url_list = utils_file.load_dict_list_from_jsonl(url_list_file)
        for url_dict in tqdm.tqdm(url_list, desc="下载中", total=len(url_list)):
            try:
                en_link = url_dict['en_link']
                cn_link = url_dict['cn_link']
                en_file_path = ""
                cn_file_path = ""
                if len(en_link) != 0:
                    suffix = en_link.split('.')[-1]
                    en_file_path = utils_file.download_file(en_link, download_dir, suffix=suffix)
                if len(cn_link) != 0:
                    suffix = cn_link.split('.')[-1]
                    cn_file_path = utils_file.download_file(cn_link, download_dir, suffix=suffix)
                url_dict['en_file_path'] = en_file_path
                url_dict['cn_file_path'] = cn_file_path
                res_list.append(url_dict)
            except Exception as e:
                utils_file.logging_print(e)
                utils_file.logging_print("下载音频失败,error:url_dict:", url_dict)
                continue
        utils_file.write_dict_list_to_jsonl(res_list, output_list_file)

    # def get_all_parson_page_url_from_a_game_page(self, game_page_url="https://voicewiki.cn/wiki/%E4%BE%A0%E7%9B%97%E7%8C%8E%E8%BD%A6%E6%89%8B%EF%BC%9A%E7%BD%AA%E6%81%B6%E9%83%BD%E5%B8%82", output_list_file='./output/voicewike/侠盗猎车手_page_url_list.list'):
    #     """"""
    #     a_xpath = "//a/@href"
    #     a_list = utils_spider.handle_xpath(utils_spider.send_request(game_page_url).text, a_xpath)
    #     res_list = []
    #     for a in a_list:
    #         if a.startswith("/wiki/"):
    #             url_i = self.url + a
    #             decode_url = unquote(url_i)
    #             res_list.append(decode_url)
    #     utils_file.write_list_to_file(res_list, output_list_file)
    #     utils_file.print_list(res_list)
    #

    def get_all_page_url_from_a_game_page(self,game_page_url="https://voicewiki.cn/wiki/%E4%BE%A0%E7%9B%97%E7%8C%8E%E8%BD%A6%E6%89%8B%EF%BC%9A%E7%BD%AA%E6%81%B6%E9%83%BD%E5%B8%82", output_list_file='./output/voicewike/侠盗猎车手_page_url_list.list'):
        """"""
        xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr/td[1]/div/div[1]/a/@href'
        xpath_2 = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr/td[2]/div/div[1]/a/@href'
        xpath_3 = '//*[@id="mw-content-text"]/div/table[3]/tbody/tr/td[1]/div/div[1]/a/@href'
        xpath_4 = '//*[@id="mw-content-text"]/div/table[4]/tbody/tr[1]/td[2]/div/div[1]/a/@href'
        xpath_5 = '//*[@id="mw-content-text"]/div/table[5]/tbody/tr[1]/td[2]/div/div[1]/a/@href'
        xpath_6 = '//*[@id="mw-content-text"]/div/table[5]/tbody/tr[2]/td[3]/div/div[1]/a/@href'
        xpath_7 = '//*[@id="mw-content-text"]/div/div/table[3]/tbody/tr[1]/td[2]/div/a/@href'
        xpath_8 = '//*[@id="mw-content-text"]/div/table[10]/tbody/tr[1]/td[2]/span/a/@href'
        page_txt = utils_spider.send_request(game_page_url).text
        num4table_xpath = '//*[@id="mw-content-text"]/div/table'
        num4table = len(utils_spider.handle_xpath(page_txt, num4table_xpath))
        res_list = []
        utils_file.logging_print('一共有: num4table:', num4table)
        for i in range(1, num4table + 1):
            num4person_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr/td'
            num4person = len(utils_spider.handle_xpath(page_txt, num4person_xpath))
            utils_file.logging_print(f'第{i}个表格有: num4person个人:', num4person)
            num4tr_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr'
            num4tr = len(utils_spider.handle_xpath(page_txt, num4tr_xpath))
            utils_file.logging_print(f'第{i}个表格有: num4tr行:', num4tr)
            num4td_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/td'
            num4td = len(utils_spider.handle_xpath(page_txt, num4td_xpath))
            utils_file.logging_print(f'第{i}个表格有: num4td列:', num4td)
            for j in range(1, num4tr + 1):
                for k in range(1, num4td + 1):
                    person_url_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[{j}]/td[{k}]/div/div[1]/a/@href'
                    person_url = utils_spider.handle_xpath(page_txt, person_url_xpath)
                    if len(person_url) != 0:
                        url = self.url + person_url[0]
                        decoded_url = unquote(url, 'utf-8')
                        res_list.append(decoded_url)
                        utils_file.logging_print(decoded_url)
        res_list = list(set(res_list))
        utils_file.write_list_to_file(res_list, output_list_file)
    def get_all_game_url(self, output_path='./output/voicewike/all_game_url.txt'):
        # 首页
        input_url = "https://voicewiki.cn/wiki/%E9%A6%96%E9%A1%B5"
        xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr[1]/td[1]/div/div/div[1]/a/@href'
        html_page = utils_spider.send_request(input_url).text
        tr_num_xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr'
        tr_num = len(utils_spider.handle_xpath(html_page, tr_num_xpath))
        td_num_xpath = '//*[@id="mw-content-text"]/div/table[2]/tbody/tr[1]/td'
        td_num = len(utils_spider.handle_xpath(html_page, td_num_xpath))
        res_list = []
        for i in range(1, tr_num + 1):
            for j in range(1, td_num + 1):
                game_url_xpath = f'//*[@id="mw-content-text"]/div/table[2]/tbody/tr[{i}]/td[{j}]/div/div/div[1]/a/@href'
                game_url = utils_spider.handle_xpath(html_page, game_url_xpath)
                if len(game_url) != 0:
                    url = self.url + game_url[0]
                    decoded_url = unquote(url, 'utf-8')
                    res_list.append(decoded_url)
                    utils_file.logging_print(decoded_url)
        utils_file.write_list_to_file(res_list, output_path)

def main():
    output_dir_root = "/home/work_nfs14/xlgeng/tts_data_pachong_high_quality/voicewiki"
    utils_file.makedir_sil(output_dir_root)
    spider = Gxl_Spider_VoiceWiki()
    utils_file.logging_print('首先得到所有游戏的url')
    all_game_revord_file = os.path.join(output_dir_root, 'all_game_url.list')
    spider.get_all_game_url(output_path=all_game_revord_file)
    utils_file.logging_print('得到所有游戏的url完成')
    utils_file.logging_print('开始得到所有游戏的所有人物页面的url')
    persons4game_record_dir = os.path.join(output_dir_root, 'persons4game')
    audio4persons_record_dir = os.path.join(output_dir_root, 'audio4persons')
    utils_file.makedir_sil(persons4game_record_dir)
    game_url_list = utils_file.load_list_file_clean(all_game_revord_file)
    for game_url in tqdm.tqdm(game_url_list,desc='游戏数目进度',total=len(game_url_list)):
        try:
            game_name = game_url.split('/')[-1]
            utils_file.logging_print('开始处理:', game_name)
            utils_file.logging_print(f'开始得到{game_name}的所有人物页面的url')
            persons4game_file = os.path.join(persons4game_record_dir, game_name + '.list')
            spider.get_all_page_url_from_a_game_page(game_page_url=game_url, output_list_file=persons4game_file)
            utils_file.logging_print(f'得到{game_name}的所有人物页面的url完成,存入:{persons4game_file}')
            person_url_list = utils_file.load_list_file_clean(persons4game_file)
            temp_audio4persons_record_dir = os.path.join(audio4persons_record_dir, game_name)
            utils_file.makedir_sil(temp_audio4persons_record_dir)
            for person_url in tqdm.tqdm(person_url_list,desc=f'{game_name}的人物数目进度',total=len(person_url_list)):
                try:
                    person_name = person_url.split('/')[-1]
                    utils_file.logging_print('开始处理:', person_name)
                    utils_file.logging_print(f'开始得到{person_name}的音频url')
                    output_person_file = os.path.join(temp_audio4persons_record_dir, person_name+'_url.jsonl')
                    spider.get_url_from_one_page(person_url, output_person_file)
                    utils_file.logging_print(f'得到{person_name}的音频url完成,存入:{output_person_file}')
                    utils_file.logging_print('开始下载所有音频:', person_name)
                    output_person_audio_file = os.path.join(temp_audio4persons_record_dir, person_name + '_file_path.jsonl')
                    download_dir_temp = os.path.join(output_dir_root, 'audio', game_name, person_name)
                    spider.start_download_from_one_page_url_file(output_person_file,download_dir_temp, output_list_file=output_person_audio_file)
                    utils_file.logging_print(f'音频下载完成处理完成:{person_name}')
                except Exception as e:
                    utils_file.logging_print('error:', e)
                    utils_file.logging_print(f'error: {person_url},处理失败,开始下一个')
                    continue

        except Exception as e:
            utils_file.logging_print('error:', e)
            utils_file.logging_print(f'error: {game_url},处理失败,开始下一个')
            continue

def main_2():
    """侠盗猎车游戏"""
    output_dir_root = "/home/work_nfs14/xlgeng/tts_data_pachong_high_quality/voicewiki_2"
    utils_file.makedir_sil(output_dir_root)
    spider = Gxl_Spider_VoiceWiki()
    game_url = "https://voicewiki.cn/wiki/%E4%BE%A0%E7%9B%97%E7%8C%8E%E8%BD%A6%E6%89%8B%EF%BC%9A%E7%BD%AA%E6%81%B6%E9%83%BD%E5%B8%82"
    game_name = game_url.split('/')[-1]
    utils_file.logging_print('开始处理:', game_name)
    utils_file.logging_print(f'开始得到{game_name}的所有人物页面的url')
    persons4game_record_dir = os.path.join(output_dir_root, 'persons4game')
    persons4game_file = os.path.join(persons4game_record_dir, game_name + '.list')
    spider.get_all_page_url_from_a_game_page(game_page_url=game_url, output_list_file=persons4game_file)
    utils_file.logging_print(f'得到{game_name}的所有人物页面的url完成,存入:{persons4game_file}')
    person_url_list = utils_file.load_list_file_clean(persons4game_file)
    audio4persons_record_dir = os.path.join(output_dir_root, 'audio4persons')
    temp_audio4persons_record_dir = os.path.join(audio4persons_record_dir, game_name)
    utils_file.makedir_sil(temp_audio4persons_record_dir)
    for person_url in tqdm.tqdm(person_url_list,desc=f'{game_name}的人物数目进度',total=len(person_url_list)):
        try:
            person_name = person_url.split('/')[-1]
            utils_file.logging_print('开始处理:', person_name)
            utils_file.logging_print(f'开始得到{person_name}的音频url')
            output_person_file = os.path.join(temp_audio4persons_record_dir, person_name+'_url.jsonl')
            spider.get_url_from_one_page(person_url, output_person_file)
            utils_file.logging_print(f'得到{person_name}的音频url完成,存入:{output_person_file}')
            utils_file.logging_print('开始下载所有音频:', person_name)
            output_person_audio_file = os.path.join(temp_audio4persons_record_dir, person_name + '_file_path.jsonl')
            download_dir_temp = os.path.join(output_dir_root, 'audio', game_name, person_name)
            spider.start_download_from_one_page_url_file(output_person_file,download_dir_temp, output_list_file=output_person_audio_file)
            utils_file.logging_print(f'音频下载完成处理完成:{person_name}')
        except Exception as e:
            utils_file.logging_print('error:', e)
            utils_file.logging_print(f'error: {person_url},处理失败,开始下一个')
            continue


class WangZhe_spider:
    def __init__(self):
        self.url = "https://voicewiki.cn"
        self.game_url = "https://voicewiki.cn/wiki/%E7%8E%8B%E8%80%85%E8%8D%A3%E8%80%80"
        self.save_dir = os.path.join(SAVE_DIR,'王者荣耀')
    def get_person_pages_url(self):
        xpath_person_page_url = '//*[@id="mw-content-text"]/div/table/tbody/tr[3]/td[2]/p/span[3]/a/@href'
        xpath =                 '//*[@id="mw-content-text"]/div/table/tbody/tr[4]/td[2]/p/span[4]/a/@href'
        xpath_22 =              '//*[@id="mw-content-text"]/div/table/tbody/tr[7]/td[2]/p/span[13]/a/@href'
        tr_num_xpath = '//*[@id="mw-content-text"]/div/table/tbody/tr'
        html_page = utils_spider.send_request(self.game_url).text
        tr_num = len(utils_spider.handle_xpath(html_page, tr_num_xpath))
        utils_file.logging_print(f'一共有{tr_num}个数据块')
        res_dict = {}
        for i in range(1,tr_num+1):
            td_num_xpath = f'//*[@id="mw-content-text"]/div/table/tbody/tr[{i}]/td[2]/p/span'
            td_num = len(utils_spider.handle_xpath(html_page, td_num_xpath))
            utils_file.logging_print(f'开始处理第{i}个数据块，这个数据块共有{td_num}个人物')
            for j in range(1,td_num+1):
                person_url_xpath = f'//*[@id="mw-content-text"]/div/table/tbody/tr[{i}]/td[2]/p/span[{j}]/a/@href'
                person_url = utils_spider.handle_xpath(html_page, person_url_xpath)[-1]
                decode_person_url = unquote(person_url)
                person_name = decode_person_url.split('/')[-1]
                res_dict[person_name] =self.url + decode_person_url
        utils_file.logging_print(f'处理完成，一共有{len(res_dict)}个人物')
        utils_file.write_dict_to_scp(res_dict,os.path.join(self.save_dir,'person_page_url.scp'))
        return res_dict

    def get_audio_url_from_person_page(self,person_page_url='https://voicewiki.cn/wiki/朵莉亚（王者荣耀）', person_name=None):
        """"""
        if person_name is None:
            person_name = person_page_url.split('/')[-1]
        html_page_tree = utils_spider.send_request(person_page_url)
        if html_page_tree is None:
            return
        html_page = html_page_tree.text
        xpath_1 = '//*[@id="mw-content-text"]/div/table[3]/tbody/tr[1]/td[1]/span/div[1]/div[2]'
        xpath_2 = '//*[@id="mw-content-text"]/div/table[3]/tbody/tr[1]/td[1]/span/div[2]/div[2]'
        xpath_3 = '//*[@id="mw-content-text"]/div/table[5]/tbody/tr[1]/td[1]/span/div[1]/div[2]'
        xpath_4 = '//*[@id="mw-content-text"]/div/table[6]/tbody/tr[1]/td[1]/span/div[1]/div[2]'
        xpath_5 = '//*[@id="mw-content-text"]/div/table[55]/tbody/tr[1]/td[1]/span/div[1]/div[2]'
        xpath_6 = '//*[@id="mw-content-text"]/div/table[55]/tbody/tr[1]/td[1]/span/div[1]/div[2]/div/div[2]/a/@href'
        xpath_8 = '//*[@id="mw-content-text"]/div/table[5]/tbody/tr[1]/td[1]/span/div[1]/div[2]/div/div[2]/a/@href'
        utils_file.logging_print('开始得到所有的音频块数')
        num_table_xpath = '//*[@id="mw-content-text"]/div/table'
        num_table = len(utils_spider.handle_xpath(html_page, num_table_xpath))
        utils_file.logging_print(f'一共有{num_table}个音频块')
        res_list = []
        for i in range(1,num_table+1):
            num_audio_xpath = '//*[@id="mw-content-text"]/div/table[5]/tbody/tr[1]/td[1]/span/div'
            num_audio = len(utils_spider.handle_xpath(html_page, num_audio_xpath))
            if num_audio == 2:
                utils_file.logging_print('这个音频块有中英两种语音')
                # 中英双语的情况
                cn_audio_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/td[1]/span/div[1]/div[2]/div/div[2]/a/@href'
                en_audio_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/td[1]/span/div[2]/div[2]/div/div[2]/a/@href'
                en_text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[1]/td[2]//text()'
                cn_text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr[2]/td//text()'
                en_audio_url = utils_spider.handle_xpath(html_page, en_audio_xpath)
                cn_audio_url = utils_spider.handle_xpath(html_page, cn_audio_xpath)
                en_text = utils_spider.handle_xpath(html_page, en_text_xpath)
                cn_text = utils_spider.handle_xpath(html_page, cn_text_xpath)
                temp_dict = {}
                if len(cn_audio_url) != 0:
                    audio_url = self.url + cn_audio_url[0]
                    decode_audio_url = unquote(audio_url, 'utf-8')
                    temp_dict['cn_link'] = decode_audio_url
                if len(en_audio_url) != 0:
                    audio_url = self.url + en_audio_url[0]
                    decode_audio_url = unquote(audio_url, 'utf-8')
                    temp_dict['en_link'] = decode_audio_url
                if len(en_text) != 0:
                    temp_dict['en_text'] = en_text[0].strip()
                if len(cn_text) != 0:
                    temp_dict['cn_text'] = cn_text[0].strip()
                if len(temp_dict) != 0:
                    res_list.append(temp_dict)
                    utils_file.logging_print(f'temp_dict不为空:{temp_dict}')
            elif num_audio == 1:
                utils_file.logging_print('这个音频块只有中文')
                # 只用中文的情况
                temp_dict = {}
                audio_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr/th/div/div[2]/div/div[2]/a/@href'
                audio_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr/th/div/div[2]/div/div[2]/a/@href'
                audio_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr//div[2]/a/@href'
                text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody//tr/td//text()'
                text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr/td//text()'
                # text_xpath = f'//*[@id="mw-content-text"]/div/table[{i}]/tbody/tr/td[2]//text()'
                audio_url = utils_spider.handle_xpath(html_page, audio_xpath)
                text = utils_spider.handle_xpath(html_page, text_xpath)
                if len(audio_url) != 0:
                    audio_url = self.url + audio_url[-1]
                    decode_audio_url = unquote(audio_url, 'utf-8')
                    temp_dict['cn_link'] = decode_audio_url
                if len(text) != 0:
                    temp_dict['cn_text'] = text[-1].strip()
                if len(temp_dict) != 0:
                    res_list.append(temp_dict)
                    utils_file.logging_print(f'temp_dict不为空:{temp_dict}')
            else:
                utils_file.logging_print('多语的情况直接舍弃')
        utils_file.write_dict_list_to_jsonl(res_list,os.path.join(self.save_dir,person_name,f'{person_name}_audio_url.jsonl'))
        return res_list

    def download_audio_from_person_page(self,dict_list, person_name):
        utils_file.logging_print('开始下载音频 for person: {}'.format(person_name))
        res_list = []
        for dict_i in tqdm.tqdm(dict_list, desc=f'{person_name}的音频下载进度', total=len(dict_list)):
            if 'en_link' in dict_i:
                en_wav_path = utils_file.download_file(dict_i['en_link'], os.path.join(self.save_dir,person_name,"audio"))
                dict_i['en_file_path'] = en_wav_path
            if 'cn_link' in dict_i:
                cn_wav_path = utils_file.download_file(dict_i['cn_link'], os.path.join(self.save_dir,person_name,"audio"))
                dict_i['cn_file_path'] = cn_wav_path
            res_list.append(dict_i)
        utils_file.write_dict_list_to_jsonl(res_list,os.path.join(self.save_dir,person_name,f'{person_name}_audio.jsonl'))

    def get_all_result_and_cat(self):
        utils_file.logging_print(f'保存路径为:{self.save_dir}')
        res_path_list = glob.glob(os.path.join(self.save_dir, '*','*_audio.jsonl'))
        utils_file.print_list(res_path_list)
        res_list = []
        for res_path in tqdm.tqdm(res_path_list, total=len(res_path_list)):
            res_list.extend(utils_file.load_list_file_clean(res_path))
        utils_file.write_list_to_file(res_list, os.path.join(self.save_dir, 'all_audio.jsonl'))
    def run(self):
        res_dict = self.get_person_pages_url()
        for person_name, person_page_url in tqdm.tqdm(res_dict.items(), desc='人物进度', total=len(res_dict)):
            try:
                utils_file.makedir(os.path.join(self.save_dir, person_name))
                dict_list = self.get_audio_url_from_person_page(person_page_url=person_page_url,
                                                                person_name=person_name)
                self.download_audio_from_person_page(dict_list, person_name=person_name)
            except Exception as e:
                utils_file.logging_print(e)
                continue


class Huangshizhanzheng_spider:
    """
    皇室战争
    """



if __name__ == '__main__':
    """"""
    # wangzherongyao = WangZhe_spider()
    # wangzherongyao.get_all_result_and_cat()
    save_dir = ''
    res_path_list = glob.glob(os.path.join("/home/work_nfs14/xlgeng/tts_data_pachong_high_quality/voicewiki/audio4persons/侠盗猎车手：罪恶都市", '*file_path.jsonl'))
    utils_file.print_list(res_path_list)
    res_list = []
    for res_path in tqdm.tqdm(res_path_list, total=len(res_path_list)):
        res_list.extend(utils_file.load_list_file_clean(res_path))
    utils_file.write_list_to_file(res_list, os.path.join('/home/work_nfs14/xlgeng/tts_data_pachong_high_quality/voicewiki/audio4persons/侠盗猎车手：罪恶都市', 'all_audio.jsonl'))


# Copyright    2021-2023  Xiaomi Corp.        (authors: Fangjun Kuang,
#                                                       Wei Kang,
#                                                       Zengwei Yao)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Tuple

import k2
import torch
import torch.nn as nn
from encoder_interface import EncoderInterface
from scaling import ScaledLinear

from icefall.utils import add_sos, make_pad_mask


class AsrModel(nn.Module):
    def __init__(
        self,
        encoder_embed: nn.Module,
        encoder: EncoderInterface,
        decoder: Optional[nn.Module] = None,
        joiner: Optional[nn.Module] = None,
        encoder_dim: int = 384,
        decoder_dim: int = 512,
        vocab_size: int = 500,
        use_transducer: bool = True,
        use_ctc: bool = False,
    ):
        """A joint CTC & Transducer ASR model.

        - Connectionist temporal classification: labelling unsegmented sequence data with recurrent neural networks (http://imagine.enpc.fr/~obozinsg/teaching/mva_gm/papers/ctc.pdf)
        - Sequence Transduction with Recurrent Neural Networks (https://arxiv.org/pdf/1211.3711.pdf)
        - Pruned RNN-T for fast, memory-efficient ASR training (https://arxiv.org/pdf/2206.13236.pdf)

        Args:
          encoder_embed:
            It is a Convolutional 2D subsampling module. It converts
            an input of shape (N, T, idim) to an output of of shape
            (N, T', odim), where T' = (T-3)//2-2 = (T-7)//2.
          encoder:
            It is the transcription network in the paper. Its accepts
            two inputs: `x` of (N, T, encoder_dim) and `x_lens` of shape (N,).
            It returns two tensors: `logits` of shape (N, T, encoder_dim) and
            `logit_lens` of shape (N,).
          decoder:
            It is the prediction network in the paper. Its input shape
            is (N, U) and its output shape is (N, U, decoder_dim).
            It should contain one attribute: `blank_id`.
            It is used when use_transducer is True.
          joiner:
            It has two inputs with shapes: (N, T, encoder_dim) and (N, U, decoder_dim).
            Its output shape is (N, T, U, vocab_size). Note that its output contains
            unnormalized probs, i.e., not processed by log-softmax.
            It is used when use_transducer is True.
          use_transducer:
            Whether use transducer head. Default: True.
          use_ctc:
            Whether use CTC head. Default: False.
        """
        super().__init__()

        assert (
            use_transducer or use_ctc
        ), f"At least one of them should be True, but got use_transducer={use_transducer}, use_ctc={use_ctc}"

        assert isinstance(encoder, EncoderInterface), type(encoder)

        self.encoder_embed = encoder_embed
        self.encoder = encoder

        self.use_transducer = use_transducer
        if use_transducer:
            # Modules for Transducer head
            assert decoder is not None
            assert hasattr(decoder, "blank_id")
            assert joiner is not None

            self.decoder = decoder
            self.joiner = joiner

            self.simple_am_proj = ScaledLinear(
                encoder_dim, vocab_size, initial_scale=0.25
            )
            self.simple_lm_proj = ScaledLinear(
                decoder_dim, vocab_size, initial_scale=0.25
            )
        else:
            assert decoder is None
            assert joiner is None

        self.use_ctc = use_ctc
        if use_ctc:
            # Modules for CTC head
            self.ctc_output = nn.Sequential(
                nn.Dropout(p=0.1),
                nn.Linear(encoder_dim, vocab_size),
                nn.LogSoftmax(dim=-1),
            )

    def forward_encoder(
        self, x: torch.Tensor, x_lens: torch.Tensor
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        """Compute encoder outputs.
        Args:
          x:
            A 3-D tensor of shape (N, T, C).
          x_lens:
            A 1-D tensor of shape (N,). It contains the number of frames in `x`
            before padding.

        Returns:
          encoder_out:
            Encoder output, of shape (N, T, C).
          encoder_out_lens:
            Encoder output lengths, of shape (N,).
        """
        # logging.info(f"Memory allocated at entry: {torch.cuda.memory_allocated() // 1000000}M")
        x, x_lens = self.encoder_embed(x, x_lens)
        # logging.info(f"Memory allocated after encoder_embed: {torch.cuda.memory_allocated() // 1000000}M")

        src_key_padding_mask = make_pad_mask(x_lens)
        x = x.permute(1, 0, 2)  # (N, T, C) -> (T, N, C)

        encoder_out, encoder_out_lens = self.encoder(x, x_lens, src_key_padding_mask)

        encoder_out = encoder_out.permute(1, 0, 2)  # (T, N, C) ->(N, T, C)
        assert torch.all(encoder_out_lens > 0), (x_lens, encoder_out_lens)

        return encoder_out, encoder_out_lens

    def forward_ctc(
        self,
        encoder_out: torch.Tensor,
        encoder_out_lens: torch.Tensor,
        targets: torch.Tensor,
        target_lengths: torch.Tensor,
    ) -> torch.Tensor:
        """Compute CTC loss.
        Args:
          encoder_out:
            Encoder output, of shape (N, T, C).
          encoder_out_lens:
            Encoder output lengths, of shape (N,).
          targets:
            Target Tensor of shape (sum(target_lengths)). The targets are assumed
            to be un-padded and concatenated within 1 dimension.
        """
        # Compute CTC log-prob
        ctc_output = self.ctc_output(encoder_out)  # (N, T, C)

        ctc_loss = torch.nn.functional.ctc_loss(
            log_probs=ctc_output.permute(1, 0, 2),  # (T, N, C)
            targets=targets.cpu(),
            input_lengths=encoder_out_lens.cpu(),
            target_lengths=target_lengths.cpu(),
            reduction="sum",
        )
        return ctc_loss

    def forward_transducer(
        self,
        encoder_out: torch.Tensor,
        encoder_out_lens: torch.Tensor,
        y: k2.RaggedTensor,
        y_lens: torch.Tensor,
        prune_range: int = 5,
        am_scale: float = 0.0,
        lm_scale: float = 0.0,
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        """Compute Transducer loss.
        Args:
          encoder_out:
            Encoder output, of shape (N, T, C).
          encoder_out_lens:
            Encoder output lengths, of shape (N,).
          y:
            A ragged tensor with 2 axes [utt][label]. It contains labels of each
            utterance.
          prune_range:
            The prune range for rnnt loss, it means how many symbols(context)
            we are considering for each frame to compute the loss.
          am_scale:
            The scale to smooth the loss with am (output of encoder network)
            part
          lm_scale:
            The scale to smooth the loss with lm (output of predictor network)
            part
        """
        # Now for the decoder, i.e., the prediction network
        blank_id = self.decoder.blank_id
        sos_y = add_sos(y, sos_id=blank_id)

        # sos_y_padded: [B, S + 1], start with SOS.
        sos_y_padded = sos_y.pad(mode="constant", padding_value=blank_id)

        # decoder_out: [B, S + 1, decoder_dim]
        decoder_out = self.decoder(sos_y_padded)

        # Note: y does not start with SOS
        # y_padded : [B, S]
        y_padded = y.pad(mode="constant", padding_value=0)

        y_padded = y_padded.to(torch.int64)
        boundary = torch.zeros(
            (encoder_out.size(0), 4),
            dtype=torch.int64,
            device=encoder_out.device,
        )
        boundary[:, 2] = y_lens
        boundary[:, 3] = encoder_out_lens

        lm = self.simple_lm_proj(decoder_out)
        am = self.simple_am_proj(encoder_out)

        # if self.training and random.random() < 0.25:
        #    lm = penalize_abs_values_gt(lm, 100.0, 1.0e-04)
        # if self.training and random.random() < 0.25:
        #    am = penalize_abs_values_gt(am, 30.0, 1.0e-04)

        with torch.cuda.amp.autocast(enabled=False):
            simple_loss, (px_grad, py_grad) = k2.rnnt_loss_smoothed(
                lm=lm.float(),
                am=am.float(),
                symbols=y_padded,
                termination_symbol=blank_id,
                lm_only_scale=lm_scale,
                am_only_scale=am_scale,
                boundary=boundary,
                reduction="sum",
                return_grad=True,
            )

        # ranges : [B, T, prune_range]
        ranges = k2.get_rnnt_prune_ranges(
            px_grad=px_grad,
            py_grad=py_grad,
            boundary=boundary,
            s_range=prune_range,
        )

        # am_pruned : [B, T, prune_range, encoder_dim]
        # lm_pruned : [B, T, prune_range, decoder_dim]
        am_pruned, lm_pruned = k2.do_rnnt_pruning(
            am=self.joiner.encoder_proj(encoder_out),
            lm=self.joiner.decoder_proj(decoder_out),
            ranges=ranges,
        )

        # logits : [B, T, prune_range, vocab_size]

        # project_input=False since we applied the decoder's input projections
        # prior to do_rnnt_pruning (this is an optimization for speed).
        logits = self.joiner(am_pruned, lm_pruned, project_input=False)

        with torch.cuda.amp.autocast(enabled=False):
            pruned_loss = k2.rnnt_loss_pruned(
                logits=logits.float(),
                symbols=y_padded,
                ranges=ranges,
                termination_symbol=blank_id,
                boundary=boundary,
                reduction="sum",
            )

        return simple_loss, pruned_loss

    def forward(
        self,
        x: torch.Tensor,
        x_lens: torch.Tensor,
        y: k2.RaggedTensor,
        prune_range: int = 5,
        am_scale: float = 0.0,
        lm_scale: float = 0.0,
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        """
        Args:
          x:
            A 3-D tensor of shape (N, T, C).
          x_lens:
            A 1-D tensor of shape (N,). It contains the number of frames in `x`
            before padding.
          y:
            A ragged tensor with 2 axes [utt][label]. It contains labels of each
            utterance.
          prune_range:
            The prune range for rnnt loss, it means how many symbols(context)
            we are considering for each frame to compute the loss.
          am_scale:
            The scale to smooth the loss with am (output of encoder network)
            part
          lm_scale:
            The scale to smooth the loss with lm (output of predictor network)
            part
        Returns:
          Return the transducer losses and CTC loss,
          in form of (simple_loss, pruned_loss, ctc_loss)

        Note:
           Regarding am_scale & lm_scale, it will make the loss-function one of
           the form:
              lm_scale * lm_probs + am_scale * am_probs +
              (1-lm_scale-am_scale) * combined_probs
        """
        assert x.ndim == 3, x.shape
        assert x_lens.ndim == 1, x_lens.shape
        assert y.num_axes == 2, y.num_axes

        assert x.size(0) == x_lens.size(0) == y.dim0, (x.shape, x_lens.shape, y.dim0)

        # Compute encoder outputs
        encoder_out, encoder_out_lens = self.forward_encoder(x, x_lens)

        row_splits = y.shape.row_splits(1)
        y_lens = row_splits[1:] - row_splits[:-1]

        if self.use_transducer:
            # Compute transducer loss
            simple_loss, pruned_loss = self.forward_transducer(
                encoder_out=encoder_out,
                encoder_out_lens=encoder_out_lens,
                y=y.to(x.device),
                y_lens=y_lens,
                prune_range=prune_range,
                am_scale=am_scale,
                lm_scale=lm_scale,
            )
        else:
            simple_loss = torch.empty(0)
            pruned_loss = torch.empty(0)

        if self.use_ctc:
            # Compute CTC loss
            targets = y.values
            ctc_loss = self.forward_ctc(
                encoder_out=encoder_out,
                encoder_out_lens=encoder_out_lens,
                targets=targets,
                target_lengths=y_lens,
            )
        else:
            ctc_loss = torch.empty(0)

        return simple_loss, pruned_loss, ctc_loss



from gxl_ai_utils.utils import utils_file
import random
input_wav = "/home/work_nfs14/xlgeng/asr_data_shard/wenetspeech_new_all/wav.scp"
input_text = "/home/work_nfs14/xlgeng/asr_data_shard/wenetspeech_new_all/text"

wav_dict = utils_file.load_dict_from_scp(input_wav)
text_dict = utils_file.load_dict_from_scp(input_text)
# 随机抽出十条
key_list = []
for i in range(10):
    key = random.choice(list(wav_dict.keys()))
    key_list.append(key)



#!/usr/bin/env python3
# Copyright 2021-2023 Xiaomi Corporation (Author: Fangjun Kuang, Zengwei Yao)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This script loads torchscript models, exported by `torch.jit.script()`
and uses them to decode waves.
You can use the following command to get the exported models:

./zipformer/export.py \
  --exp-dir ./zipformer/exp \
  --tokens data/lang_bpe_500/tokens.txt \
  --epoch 30 \
  --avg 9 \
  --jit 1

Usage of this script:

./zipformer/jit_pretrained.py \
  --nn-model-filename ./zipformer/exp/cpu_jit.pt \
  --tokens ./data/lang_bpe_500/tokens.txt \
  /path/to/foo.wav \
  /path/to/bar.wav
"""

import argparse
import logging
import math
from typing import List

import k2
import kaldifeat
import torch
import torchaudio
from torch.nn.utils.rnn import pad_sequence


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--nn-model-filename",
        type=str,
        required=True,
        help="Path to the torchscript model cpu_jit.pt",
    )

    parser.add_argument(
        "--tokens",
        type=str,
        help="""Path to tokens.txt.""",
    )

    parser.add_argument(
        "sound_files",
        type=str,
        nargs="+",
        help="The input sound file(s) to transcribe. "
        "Supported formats are those supported by torchaudio.load(). "
        "For example, wav and flac are supported. "
        "The sample rate has to be 16kHz.",
    )

    return parser


def read_sound_files(
    filenames: List[str], expected_sample_rate: float = 16000
) -> List[torch.Tensor]:
    """Read a list of sound files into a list 1-D float32 torch tensors.
    Args:
      filenames:
        A list of sound filenames.
      expected_sample_rate:
        The expected sample rate of the sound files.
    Returns:
      Return a list of 1-D float32 torch tensors.
    """
    ans = []
    for f in filenames:
        wave, sample_rate = torchaudio.load(f)
        assert (
            sample_rate == expected_sample_rate
        ), f"expected sample rate: {expected_sample_rate}. Given: {sample_rate}"
        # We use only the first channel
        ans.append(wave[0].contiguous())
    return ans


def greedy_search(
    model: torch.jit.ScriptModule,
    encoder_out: torch.Tensor,
    encoder_out_lens: torch.Tensor,
) -> List[List[int]]:
    """Greedy search in batch mode. It hardcodes --max-sym-per-frame=1.
    Args:
      model:
        The transducer model.
      encoder_out:
        A 3-D tensor of shape (N, T, C)
      encoder_out_lens:
        A 1-D tensor of shape (N,).
    Returns:
      Return the decoded results for each utterance.
    """
    assert encoder_out.ndim == 3
    assert encoder_out.size(0) >= 1, encoder_out.size(0)

    packed_encoder_out = torch.nn.utils.rnn.pack_padded_sequence(
        input=encoder_out,
        lengths=encoder_out_lens.cpu(),
        batch_first=True,
        enforce_sorted=False,
    )

    device = encoder_out.device
    blank_id = model.decoder.blank_id

    batch_size_list = packed_encoder_out.batch_sizes.tolist()
    N = encoder_out.size(0)

    assert torch.all(encoder_out_lens > 0), encoder_out_lens
    assert N == batch_size_list[0], (N, batch_size_list)

    context_size = model.decoder.context_size
    hyps = [[blank_id] * context_size for _ in range(N)]

    decoder_input = torch.tensor(
        hyps,
        device=device,
        dtype=torch.int64,
    )  # (N, context_size)

    decoder_out = model.decoder(
        decoder_input,
        need_pad=torch.tensor([False]),
    ).squeeze(1)

    offset = 0
    for batch_size in batch_size_list:
        start = offset
        end = offset + batch_size
        current_encoder_out = packed_encoder_out.data[start:end]
        current_encoder_out = current_encoder_out
        # current_encoder_out's shape: (batch_size, encoder_out_dim)
        offset = end

        decoder_out = decoder_out[:batch_size]

        logits = model.joiner(
            current_encoder_out,
            decoder_out,
        )
        # logits'shape (batch_size, vocab_size)

        assert logits.ndim == 2, logits.shape
        y = logits.argmax(dim=1).tolist()
        emitted = False
        for i, v in enumerate(y):
            if v != blank_id:
                hyps[i].append(v)
                emitted = True
        if emitted:
            # update decoder output
            decoder_input = [h[-context_size:] for h in hyps[:batch_size]]
            decoder_input = torch.tensor(
                decoder_input,
                device=device,
                dtype=torch.int64,
            )
            decoder_out = model.decoder(
                decoder_input,
                need_pad=torch.tensor([False]),
            )
            decoder_out = decoder_out.squeeze(1)

    sorted_ans = [h[context_size:] for h in hyps]
    ans = []
    unsorted_indices = packed_encoder_out.unsorted_indices.tolist()
    for i in range(N):
        ans.append(sorted_ans[unsorted_indices[i]])

    return ans


@torch.no_grad()
def main():
    parser = get_parser()
    args = parser.parse_args()
    logging.info(vars(args))

    device = torch.device("cpu")
    if torch.cuda.is_available():
        device = torch.device("cuda", 0)

    logging.info(f"device: {device}")

    model = torch.jit.load(args.nn_model_filename)

    model.eval()

    model.to(device)

    logging.info("Constructing Fbank computer")
    opts = kaldifeat.FbankOptions()
    opts.device = device
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False
    opts.frame_opts.samp_freq = 16000
    opts.mel_opts.num_bins = 80
    opts.mel_opts.high_freq = -400

    fbank = kaldifeat.Fbank(opts)

    logging.info(f"Reading sound files: {args.sound_files}")
    waves = read_sound_files(
        filenames=args.sound_files,
    )
    waves = [w.to(device) for w in waves]

    logging.info("Decoding started")
    features = fbank(waves)
    feature_lengths = [f.size(0) for f in features]

    features = pad_sequence(
        features,
        batch_first=True,
        padding_value=math.log(1e-10),
    )

    feature_lengths = torch.tensor(feature_lengths, device=device)

    encoder_out, encoder_out_lens = model.encoder(
        features=features,
        feature_lengths=feature_lengths,
    )

    hyps = greedy_search(
        model=model,
        encoder_out=encoder_out,
        encoder_out_lens=encoder_out_lens,
    )

    s = "\n"

    token_table = k2.SymbolTable.from_file(args.tokens)

    def token_ids_to_words(token_ids: List[int]) -> str:
        text = ""
        for i in token_ids:
            text += token_table[i]
        return text.replace("▁", " ").strip()

    for filename, hyp in zip(args.sound_files, hyps):
        words = token_ids_to_words(hyp)
        s += f"{filename}:\n{words}\n"

    logging.info(s)

    logging.info("Decoding Done")


if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



#!/usr/bin/env python3
# Copyright      2022  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This script loads ONNX models and uses them to decode waves.
You can use the following command to get the exported models:

We use the pre-trained model from
https://huggingface.co/Zengwei/icefall-asr-librispeech-zipformer-2023-05-15
as an example to show how to use this file.

1. Download the pre-trained model

cd egs/librispeech/ASR

repo_url=https://huggingface.co/Zengwei/icefall-asr-librispeech-zipformer-2023-05-15
GIT_LFS_SKIP_SMUDGE=1 git clone $repo_url
repo=$(basename $repo_url)

pushd $repo
git lfs pull --include "exp/pretrained.pt"

cd exp
ln -s pretrained.pt epoch-99.pt
popd

2. Export the model to ONNX

./zipformer/export-onnx.py \
  --tokens $repo/data/lang_bpe_500/tokens.txt \
  --use-averaged-model 0 \
  --epoch 99 \
  --avg 1 \
  --exp-dir $repo/exp \
  --causal False

It will generate the following 3 files inside $repo/exp:

  - encoder-epoch-99-avg-1.onnx
  - decoder-epoch-99-avg-1.onnx
  - joiner-epoch-99-avg-1.onnx

3. Run this file

./zipformer/onnx_pretrained.py \
  --encoder-model-filename $repo/exp/encoder-epoch-99-avg-1.onnx \
  --decoder-model-filename $repo/exp/decoder-epoch-99-avg-1.onnx \
  --joiner-model-filename $repo/exp/joiner-epoch-99-avg-1.onnx \
  --tokens $repo/data/lang_bpe_500/tokens.txt \
  $repo/test_wavs/1089-134686-0001.wav \
  $repo/test_wavs/1221-135766-0001.wav \
  $repo/test_wavs/1221-135766-0002.wav
"""

import argparse
import logging
import math
from typing import List, Tuple

import k2
import kaldifeat
import onnxruntime as ort
import torch
import torchaudio
from torch.nn.utils.rnn import pad_sequence


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--encoder-model-filename",
        type=str,
        required=True,
        help="Path to the encoder onnx model. ",
    )

    parser.add_argument(
        "--decoder-model-filename",
        type=str,
        required=True,
        help="Path to the decoder onnx model. ",
    )

    parser.add_argument(
        "--joiner-model-filename",
        type=str,
        required=True,
        help="Path to the joiner onnx model. ",
    )

    parser.add_argument(
        "--tokens",
        type=str,
        help="""Path to tokens.txt.""",
    )

    parser.add_argument(
        "sound_files",
        type=str,
        nargs="+",
        help="The input sound file(s) to transcribe. "
        "Supported formats are those supported by torchaudio.load(). "
        "For example, wav and flac are supported. "
        "The sample rate has to be 16kHz.",
    )

    parser.add_argument(
        "--sample-rate",
        type=int,
        default=16000,
        help="The sample rate of the input sound file",
    )

    return parser


class OnnxModel:
    def __init__(
        self,
        encoder_model_filename: str,
        decoder_model_filename: str,
        joiner_model_filename: str,
    ):
        session_opts = ort.SessionOptions()
        session_opts.inter_op_num_threads = 1
        session_opts.intra_op_num_threads = 4

        self.session_opts = session_opts

        self.init_encoder(encoder_model_filename)
        self.init_decoder(decoder_model_filename)
        self.init_joiner(joiner_model_filename)

    def init_encoder(self, encoder_model_filename: str):
        self.encoder = ort.InferenceSession(
            encoder_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )

    def init_decoder(self, decoder_model_filename: str):
        self.decoder = ort.InferenceSession(
            decoder_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )

        decoder_meta = self.decoder.get_modelmeta().custom_metadata_map
        self.context_size = int(decoder_meta["context_size"])
        self.vocab_size = int(decoder_meta["vocab_size"])

        logging.info(f"context_size: {self.context_size}")
        logging.info(f"vocab_size: {self.vocab_size}")

    def init_joiner(self, joiner_model_filename: str):
        self.joiner = ort.InferenceSession(
            joiner_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )

        joiner_meta = self.joiner.get_modelmeta().custom_metadata_map
        self.joiner_dim = int(joiner_meta["joiner_dim"])

        logging.info(f"joiner_dim: {self.joiner_dim}")

    def run_encoder(
        self,
        x: torch.Tensor,
        x_lens: torch.Tensor,
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        """
        Args:
          x:
            A 3-D tensor of shape (N, T, C)
          x_lens:
            A 2-D tensor of shape (N,). Its dtype is torch.int64
        Returns:
          Return a tuple containing:
            - encoder_out, its shape is (N, T', joiner_dim)
            - encoder_out_lens, its shape is (N,)
        """
        out = self.encoder.run(
            [
                self.encoder.get_outputs()[0].name,
                self.encoder.get_outputs()[1].name,
            ],
            {
                self.encoder.get_inputs()[0].name: x.numpy(),
                self.encoder.get_inputs()[1].name: x_lens.numpy(),
            },
        )
        return torch.from_numpy(out[0]), torch.from_numpy(out[1])

    def run_decoder(self, decoder_input: torch.Tensor) -> torch.Tensor:
        """
        Args:
          decoder_input:
            A 2-D tensor of shape (N, context_size)
        Returns:
          Return a 2-D tensor of shape (N, joiner_dim)
        """
        out = self.decoder.run(
            [self.decoder.get_outputs()[0].name],
            {self.decoder.get_inputs()[0].name: decoder_input.numpy()},
        )[0]

        return torch.from_numpy(out)

    def run_joiner(
        self, encoder_out: torch.Tensor, decoder_out: torch.Tensor
    ) -> torch.Tensor:
        """
        Args:
          encoder_out:
            A 2-D tensor of shape (N, joiner_dim)
          decoder_out:
            A 2-D tensor of shape (N, joiner_dim)
        Returns:
          Return a 2-D tensor of shape (N, vocab_size)
        """
        out = self.joiner.run(
            [self.joiner.get_outputs()[0].name],
            {
                self.joiner.get_inputs()[0].name: encoder_out.numpy(),
                self.joiner.get_inputs()[1].name: decoder_out.numpy(),
            },
        )[0]

        return torch.from_numpy(out)


def read_sound_files(
    filenames: List[str], expected_sample_rate: float
) -> List[torch.Tensor]:
    """Read a list of sound files into a list 1-D float32 torch tensors.
    Args:
      filenames:
        A list of sound filenames.
      expected_sample_rate:
        The expected sample rate of the sound files.
    Returns:
      Return a list of 1-D float32 torch tensors.
    """
    ans = []
    for f in filenames:
        wave, sample_rate = torchaudio.load(f)
        assert (
            sample_rate == expected_sample_rate
        ), f"expected sample rate: {expected_sample_rate}. Given: {sample_rate}"
        # We use only the first channel
        ans.append(wave[0])
    return ans


def greedy_search(
    model: OnnxModel,
    encoder_out: torch.Tensor,
    encoder_out_lens: torch.Tensor,
) -> List[List[int]]:
    """Greedy search in batch mode. It hardcodes --max-sym-per-frame=1.
    Args:
      model:
        The transducer model.
      encoder_out:
        A 3-D tensor of shape (N, T, joiner_dim)
      encoder_out_lens:
        A 1-D tensor of shape (N,).
    Returns:
      Return the decoded results for each utterance.
    """
    assert encoder_out.ndim == 3, encoder_out.shape
    assert encoder_out.size(0) >= 1, encoder_out.size(0)

    packed_encoder_out = torch.nn.utils.rnn.pack_padded_sequence(
        input=encoder_out,
        lengths=encoder_out_lens.cpu(),
        batch_first=True,
        enforce_sorted=False,
    )

    blank_id = 0  # hard-code to 0

    batch_size_list = packed_encoder_out.batch_sizes.tolist()
    N = encoder_out.size(0)

    assert torch.all(encoder_out_lens > 0), encoder_out_lens
    assert N == batch_size_list[0], (N, batch_size_list)

    context_size = model.context_size
    hyps = [[blank_id] * context_size for _ in range(N)]

    decoder_input = torch.tensor(
        hyps,
        dtype=torch.int64,
    )  # (N, context_size)

    decoder_out = model.run_decoder(decoder_input)

    offset = 0
    for batch_size in batch_size_list:
        start = offset
        end = offset + batch_size
        current_encoder_out = packed_encoder_out.data[start:end]
        # current_encoder_out's shape: (batch_size, joiner_dim)
        offset = end

        decoder_out = decoder_out[:batch_size]
        logits = model.run_joiner(current_encoder_out, decoder_out)

        # logits'shape (batch_size, vocab_size)

        assert logits.ndim == 2, logits.shape
        y = logits.argmax(dim=1).tolist()
        emitted = False
        for i, v in enumerate(y):
            if v != blank_id:
                hyps[i].append(v)
                emitted = True
        if emitted:
            # update decoder output
            decoder_input = [h[-context_size:] for h in hyps[:batch_size]]
            decoder_input = torch.tensor(
                decoder_input,
                dtype=torch.int64,
            )
            decoder_out = model.run_decoder(decoder_input)

    sorted_ans = [h[context_size:] for h in hyps]
    ans = []
    unsorted_indices = packed_encoder_out.unsorted_indices.tolist()
    for i in range(N):
        ans.append(sorted_ans[unsorted_indices[i]])

    return ans


@torch.no_grad()
def main():
    parser = get_parser()
    args = parser.parse_args()
    logging.info(vars(args))
    model = OnnxModel(
        encoder_model_filename=args.encoder_model_filename,
        decoder_model_filename=args.decoder_model_filename,
        joiner_model_filename=args.joiner_model_filename,
    )

    logging.info("Constructing Fbank computer")
    opts = kaldifeat.FbankOptions()
    opts.device = "cpu"
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False
    opts.frame_opts.samp_freq = args.sample_rate
    opts.mel_opts.num_bins = 80
    opts.mel_opts.high_freq = -400

    fbank = kaldifeat.Fbank(opts)

    logging.info(f"Reading sound files: {args.sound_files}")
    waves = read_sound_files(
        filenames=args.sound_files,
        expected_sample_rate=args.sample_rate,
    )

    logging.info("Decoding started")
    features = fbank(waves)
    feature_lengths = [f.size(0) for f in features]

    features = pad_sequence(
        features,
        batch_first=True,
        padding_value=math.log(1e-10),
    )

    feature_lengths = torch.tensor(feature_lengths, dtype=torch.int64)
    encoder_out, encoder_out_lens = model.run_encoder(features, feature_lengths)

    hyps = greedy_search(
        model=model,
        encoder_out=encoder_out,
        encoder_out_lens=encoder_out_lens,
    )
    s = "\n"

    token_table = k2.SymbolTable.from_file(args.tokens)

    def token_ids_to_words(token_ids: List[int]) -> str:
        text = ""
        for i in token_ids:
            text += token_table[i]
        return text.replace("▁", " ").strip()

    for filename, hyp in zip(args.sound_files, hyps):
        words = token_ids_to_words(hyp)
        s += f"{filename}:\n{words}\n"
    logging.info(s)

    logging.info("Decoding Done")


if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



from gxl_ai_utils.utils import utils_file
import os
import tqdm

test_dir = "/home/work_nfs8/xlgeng/data/scp_test"

dataset_names = os.listdir(test_dir)
def little_func(dataset_name):
    dataset_path = os.path.join(test_dir, dataset_name)
    wav_scp_path = os.path.join(dataset_path, "wav.scp")
    if not os.path.exists(wav_scp_path):
        return
    wav_dict = utils_file.load_dict_from_scp(wav_scp_path)
    wav_output_dir = os.path.join(dataset_path, "wav")
    utils_file.makedir_sil(wav_output_dir)
    new_wav_dict = {}
    for key in tqdm.tqdm(wav_dict.keys()):
        wav_path = wav_dict[key]
        new_wav_path = os.path.join(wav_output_dir, os.path.basename(wav_path))
        new_wav_dict[key] = new_wav_path
        utils_file.copy_file(wav_path, new_wav_path, use_shell=True)
    utils_file.write_dict_to_scp(new_wav_dict, wav_scp_path)
    utils_file.write_dict_to_scp(wav_dict, os.path.join(dataset_path, "wav_old.scp"))
    utils_file.do_convert_wav_text_scp_to_jsonl(os.path.join(dataset_path, "wav.scp"),os.path.join(dataset_path, "text"), os.path.join(dataset_path, "data.list"))

runner = utils_file.GxlDynamicThreadPool()
for dataset_name in dataset_names:
    runner.add_thread(little_func, [dataset_name])

runner.start()



# 将400w条贴吧问答数据进行处理, 这个数据的特点是问句很短, 回答很长,我们只取前一句话的回答.
# 具体规则如下:
# 遇到句号,且一句话内汉字个数在60个一下,则选择它. 否则,将其过滤.
import re
import emoji
import tqdm

from gxl_ai_utils.utils import utils_file

def do_get_first_sentence(s, char_num=80):
    """
    以中文句号作为分割符
    获取长字符串中符合条件的第一句话，如果满足条件（以句号结尾且汉字个数小于80）返回这句话内容和True，否则返回空字符串和False
    """
    for index in range(len(s)):
        if s[index] == '。':
            sentence = s[:index + 1].strip()
            sentence_cleaned = do_clean_chat_text(sentence)
            chinese_count = sum(1 for char in sentence_cleaned if '\u4e00' <= char <= '\u9fff')
            if chinese_count < char_num:
                return sentence_cleaned, True
            else:
                return "", False
    total_cleaned = do_clean_chat_text(s)
    chinese_count = sum(1 for char in total_cleaned if '\u4e00' <= char <= '\u9fff')
    if chinese_count < char_num:
        return total_cleaned, True
    return "", False

def do_clean_chat_text(text):
    # 去除HTML/XML标签
    text = re.sub('<.*?>', '', text)
    # 去除圆括号内内容
    text = re.sub(r'\(.*?\)', '', text)
    # 去除方括号内内容
    text = re.sub(r'\[.*?\]', '', text)
    # 去除花括号内内容
    text = re.sub(r'\{.*?\}', '', text)
    # 去除特殊字符和符号（示例去除换行符、回车符等，可按需扩展）
    text = text.replace('\n', '')
    text = text.replace('\r', '')
    # 去除多余的空格
    text = text.strip().replace("  ", " ")
    # 处理表情符号（将表情符号替换为空字符串）
    text = emoji.replace_emoji(text, replace='')
    # 过滤广告和无关链接（示例去除以http://或https://开头的链接，可按需扩展）
    text = re.sub('https?://\S+', '', text)

    return text


def main():
    # 加载jsonl
    jsonl_path = "/home/work_nfs15/asr_data/data/chat_data/web_text_2019_text/web_text_zh_train.json"
    dict_list = utils_file.load_dict_list_from_jsonl(jsonl_path)
    res_dict_list = []
    for dict_item in tqdm.tqdm(dict_list, total=len(dict_list)):
        key = dict_item["qid"]
        Q = dict_item["title"]
        A = dict_item["content"]
        A_first_sentence, if_exit = do_get_first_sentence(A)
        if if_exit:
            res_dict_list.append({"key": key, "Q": Q, "A": A_first_sentence})
        else:
            continue
    output_path = "/home/work_nfs15/asr_data/data/chat_data/web_text_2019_text/web_text_zh_train_by_gxl.jsonl"
    utils_file.write_dict_list_to_jsonl(res_dict_list, output_path)

if __name__ == '__main__':
    main()


# Copyright (c) 2022 Yifan Peng (Carnegie Mellon University)
#               2023 Voicecomm Inc (Kai Li)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Modified from ESPnet(https://github.com/espnet/espnet)
"""BranchformerEncoderLayer definition."""

import torch
import torch.nn as nn
from typing import Optional, Tuple


class BranchformerEncoderLayer(torch.nn.Module):
    """Branchformer encoder layer module.

    Args:
        size (int): model dimension
        attn: standard self-attention or efficient attention, optional
        cgmlp: ConvolutionalGatingMLP, optional
        dropout_rate (float): dropout probability
        merge_method (str): concat, learned_ave, fixed_ave
        cgmlp_weight (float): weight of the cgmlp branch, between 0 and 1,
            used if merge_method is fixed_ave
        attn_branch_drop_rate (float): probability of dropping the attn branch,
            used if merge_method is learned_ave
        stochastic_depth_rate (float): stochastic depth probability
    """

    def __init__(
        self,
        size: int,
        attn: Optional[torch.nn.Module],
        cgmlp: Optional[torch.nn.Module],
        dropout_rate: float,
        merge_method: str,
        cgmlp_weight: float = 0.5,
        attn_branch_drop_rate: float = 0.0,
        stochastic_depth_rate: float = 0.0,
    ):
        super().__init__()
        assert (attn is not None) or (
            cgmlp is not None), "At least one branch should be valid"

        self.size = size
        self.attn = attn
        self.cgmlp = cgmlp
        self.merge_method = merge_method
        self.cgmlp_weight = cgmlp_weight
        self.attn_branch_drop_rate = attn_branch_drop_rate
        self.stochastic_depth_rate = stochastic_depth_rate
        self.use_two_branches = (attn is not None) and (cgmlp is not None)

        if attn is not None:
            self.norm_mha = nn.LayerNorm(size)  # for the MHA module
        if cgmlp is not None:
            self.norm_mlp = nn.LayerNorm(size)  # for the MLP module
        self.norm_final = nn.LayerNorm(
            size)  # for the final output of the block

        self.dropout = torch.nn.Dropout(dropout_rate)

        # # attention-based pooling for two branches
        self.pooling_proj1 = torch.nn.Linear(size, 1)
        self.pooling_proj2 = torch.nn.Linear(size, 1)

        # # linear projections for calculating merging weights
        self.weight_proj1 = torch.nn.Linear(size, 1)
        self.weight_proj2 = torch.nn.Linear(size, 1)

        if self.use_two_branches:
            if self.merge_method == "concat":
                self.merge_proj = torch.nn.Linear(size + size, size)

            elif self.merge_method == "learned_ave":
                # linear projection after weighted average
                self.merge_proj = torch.nn.Linear(size, size)

            elif self.merge_method == "fixed_ave":
                assert (0.0 <= cgmlp_weight <=
                        1.0), "cgmlp weight should be between 0.0 and 1.0"

                # remove the other branch if only one branch is used
                if cgmlp_weight == 0.0:
                    self.use_two_branches = False
                    self.cgmlp = None
                    self.norm_mlp = None
                elif cgmlp_weight == 1.0:
                    self.use_two_branches = False
                    self.attn = None
                    self.norm_mha = None

                # linear projection after weighted average
                self.merge_proj = torch.nn.Linear(size, size)
            else:
                raise ValueError(f"unknown merge method: {merge_method}")
        else:
            self.merge_proj = torch.nn.Identity()

    def forward(
        self,
        x: torch.Tensor,
        mask: torch.Tensor,
        pos_emb: torch.Tensor,
        mask_pad: torch.Tensor = torch.ones((0, 0, 0), dtype=torch.bool),
        att_cache: torch.Tensor = torch.zeros((0, 0, 0, 0)),
        cnn_cache: torch.Tensor = torch.zeros((0, 0, 0, 0)),
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor]:
        """Compute encoded features.

        Args:
            x (Union[Tuple, torch.Tensor]): Input tensor  (#batch, time, size).
            mask (torch.Tensor): Mask tensor for the input (#batch, time, time).
            pos_emb (torch.Tensor): positional encoding, must not be None
                for BranchformerEncoderLayer.
            mask_pad (torch.Tensor): batch padding mask used for conv module.
                (#batch, 1，time), (0, 0, 0) means fake mask.
            att_cache (torch.Tensor): Cache tensor of the KEY & VALUE
                (#batch=1, head, cache_t1, d_k * 2), head * d_k == size.
            cnn_cache (torch.Tensor): Convolution cache in cgmlp layer
                (#batch=1, size, cache_t2)

        Returns:
            torch.Tensor: Output tensor (#batch, time, size).
            torch.Tensor: Mask tensor (#batch, time, time.
            torch.Tensor: att_cache tensor,
                (#batch=1, head, cache_t1 + time, d_k * 2).
            torch.Tensor: cnn_cahce tensor (#batch, size, cache_t2).
        """

        stoch_layer_coeff = 1.0
        skip_layer = False
        # with stochastic depth, residual connection `x + f(x)` becomes
        # `x <- x + 1 / (1 - p) * f(x)` at training time.
        if self.training and self.stochastic_depth_rate > 0:
            skip_layer = torch.rand(1).item() < self.stochastic_depth_rate
            stoch_layer_coeff = 1.0 / (1 - self.stochastic_depth_rate)

        if skip_layer:
            return x, mask, att_cache, cnn_cache

        # Two branches
        x1 = x
        x2 = x

        # Branch 1: multi-headed attention module
        if self.attn is not None:
            x1 = self.norm_mha(x1)
            x_att, new_att_cache = self.attn(x1, x1, x1, mask, pos_emb,
                                             att_cache)
            x1 = self.dropout(x_att)

        # Branch 2: convolutional gating mlp
        # Fake new cnn cache here, and then change it in conv_module
        new_cnn_cache = torch.zeros((0, 0, 0), dtype=x.dtype, device=x.device)
        if self.cgmlp is not None:
            x2 = self.norm_mlp(x2)
            x2, new_cnn_cache = self.cgmlp(x2, mask_pad, cnn_cache)
            x2 = self.dropout(x2)

        # Merge two branches
        if self.use_two_branches:
            if self.merge_method == "concat":
                x = x + stoch_layer_coeff * self.dropout(
                    self.merge_proj(torch.cat([x1, x2], dim=-1)))
            elif self.merge_method == "learned_ave":
                if (self.training and self.attn_branch_drop_rate > 0
                        and torch.rand(1).item() < self.attn_branch_drop_rate):
                    # Drop the attn branch
                    w1, w2 = torch.tensor(0.0), torch.tensor(1.0)
                else:
                    # branch1
                    score1 = (self.pooling_proj1(x1).transpose(1, 2) /
                              self.size**0.5)
                    score1 = score1.masked_fill(mask_pad.eq(0), -float('inf'))
                    score1 = torch.softmax(score1, dim=-1).masked_fill(
                        mask_pad.eq(0), 0.0)

                    pooled1 = torch.matmul(score1,
                                           x1).squeeze(1)  # (batch, size)
                    weight1 = self.weight_proj1(pooled1)  # (batch, 1)

                    # branch2
                    score2 = (self.pooling_proj2(x2).transpose(1, 2) /
                              self.size**0.5)
                    score2 = score2.masked_fill(mask_pad.eq(0), -float('inf'))
                    score2 = torch.softmax(score2, dim=-1).masked_fill(
                        mask_pad.eq(0), 0.0)

                    pooled2 = torch.matmul(score2,
                                           x2).squeeze(1)  # (batch, size)
                    weight2 = self.weight_proj2(pooled2)  # (batch, 1)

                    # normalize weights of two branches
                    merge_weights = torch.softmax(torch.cat([weight1, weight2],
                                                            dim=-1),
                                                  dim=-1)  # (batch, 2)
                    merge_weights = merge_weights.unsqueeze(-1).unsqueeze(
                        -1)  # (batch, 2, 1, 1)
                    w1, w2 = merge_weights[:,
                                           0], merge_weights[:,
                                                             1]  # (batch, 1, 1)

                x = x + stoch_layer_coeff * self.dropout(
                    self.merge_proj(w1 * x1 + w2 * x2))
            elif self.merge_method == "fixed_ave":
                x = x + stoch_layer_coeff * self.dropout(
                    self.merge_proj((1.0 - self.cgmlp_weight) * x1 +
                                    self.cgmlp_weight * x2))
            else:
                raise RuntimeError(
                    f"unknown merge method: {self.merge_method}")
        else:
            if self.attn is None:
                x = x + stoch_layer_coeff * self.dropout(self.merge_proj(x2))
            elif self.cgmlp is None:
                x = x + stoch_layer_coeff * self.dropout(self.merge_proj(x1))
            else:
                # This should not happen
                raise RuntimeError(
                    "Both branches are not None, which is unexpected.")

        x = self.norm_final(x)

        return x, mask, new_att_cache, new_cnn_cache



#!/usr/bin/env python3
#
# Copyright 2023 Xiaomi Corporation

"""
Usage:

./check-onnx-streaming.py \
  --jit ./icefall-librispeech-rnn-lm/exp/cpu_jit.pt \
  --onnx ./icefall-librispeech-rnn-lm/exp/with-state-epoch-99-avg-1.onnx

Note: You can download pre-trained models from
https://huggingface.co/ezerhouni/icefall-librispeech-rnn-lm

"""

import argparse
import logging
from typing import Tuple

import onnxruntime as ort
import torch


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--jit",
        required=True,
        type=str,
        help="Path to the torchscript model",
    )

    parser.add_argument(
        "--onnx",
        required=True,
        type=str,
        help="Path to the onnx model",
    )

    return parser


class OnnxModel:
    def __init__(self, filename: str):
        session_opts = ort.SessionOptions()
        session_opts.inter_op_num_threads = 1
        session_opts.intra_op_num_threads = 1

        self.model = ort.InferenceSession(
            filename,
            sess_options=session_opts,
        )

        meta_data = self.model.get_modelmeta().custom_metadata_map
        self.sos_id = int(meta_data["sos_id"])
        self.eos_id = int(meta_data["eos_id"])
        self.vocab_size = int(meta_data["vocab_size"])
        self.num_layers = int(meta_data["num_layers"])
        self.hidden_size = int(meta_data["hidden_size"])
        print(meta_data)

    def __call__(
        self, x: torch.Tensor, y: torch.Tensor, h0: torch.Tensor, c0: torch.Tensor
    ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
        out = self.model.run(
            [
                self.model.get_outputs()[0].name,
                self.model.get_outputs()[1].name,
                self.model.get_outputs()[2].name,
            ],
            {
                self.model.get_inputs()[0].name: x.numpy(),
                self.model.get_inputs()[1].name: y.numpy(),
                self.model.get_inputs()[2].name: h0.numpy(),
                self.model.get_inputs()[3].name: c0.numpy(),
            },
        )
        return (
            torch.from_numpy(out[0]),
            torch.from_numpy(out[1]),
            torch.from_numpy(out[2]),
        )


@torch.no_grad()
def main():
    args = get_parser().parse_args()
    logging.info(vars(args))

    torch_model = torch.jit.load(args.jit).cpu()
    onnx_model = OnnxModel(args.onnx)
    N = torch.arange(1, 5).tolist()

    num_layers = onnx_model.num_layers
    hidden_size = onnx_model.hidden_size

    for n in N:
        L = torch.randint(low=1, high=100, size=(1,)).item()
        x = torch.randint(
            low=1, high=onnx_model.vocab_size, size=(n, L), dtype=torch.int64
        )
        h0 = torch.rand(num_layers, n, hidden_size)
        c0 = torch.rand(num_layers, n, hidden_size)

        torch_log_prob, torch_h0, torch_c0 = torch_model.score_token_onnx(x, h0, c0)
        onnx_log_prob, onnx_h0, onnx_c0 = onnx_model(x, h0, c0)

        for torch_v, onnx_v in zip(
            (torch_log_prob, torch_h0, torch_c0), (onnx_log_prob, onnx_h0, onnx_c0)
        ):
            assert torch.allclose(torch_v, onnx_v, atol=1e-5), (
                torch_v.shape,
                onnx_v.shape,
                (torch_v - onnx_v).abs().max(),
            )
            print(n, L, torch_v.sum(), onnx_v.sum())


if __name__ == "__main__":
    torch.manual_seed(20230423)
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



# Copyright      2021  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Dict, List, Tuple

import torch
from torch.nn.utils.rnn import pad_sequence


def save_alignments(
    alignments: Dict[str, List[int]],
    subsampling_factor: int,
    filename: str,
) -> None:
    """Save alignments to a file.

    Args:
      alignments:
        A dict containing alignments. Keys of the dict are utterances and
        values are the corresponding framewise alignments after subsampling.
      subsampling_factor:
        The subsampling factor of the model.
      filename:
        Path to save the alignments.
    Returns:
      Return None.
    """
    ali_dict = {
        "subsampling_factor": subsampling_factor,
        "alignments": alignments,
    }
    torch.save(ali_dict, filename)


def load_alignments(filename: str) -> Tuple[int, Dict[str, List[int]]]:
    """Load alignments from a file.

    Args:
      filename:
        Path to the file containing alignment information.
        The file should be saved by :func:`save_alignments`.
    Returns:
      Return a tuple containing:
        - subsampling_factor: The subsampling_factor used to compute
          the alignments.
        - alignments: A dict containing utterances and their corresponding
          framewise alignment, after subsampling.
    """
    ali_dict = torch.load(filename)
    subsampling_factor = ali_dict["subsampling_factor"]
    alignments = ali_dict["alignments"]
    return subsampling_factor, alignments


def convert_alignments_to_tensor(
    alignments: Dict[str, List[int]], device: torch.device
) -> Dict[str, torch.Tensor]:
    """Convert alignments from list of int to a 1-D torch.Tensor.

    Args:
      alignments:
        A dict containing alignments. Keys are utterance IDs and
        values are their corresponding frame-wise alignments.
      device:
        The device to move the alignments to.
    Returns:
      Return a dict using 1-D torch.Tensor to store the alignments.
      The dtype of the tensor are `torch.int64`. We choose `torch.int64`
      because `torch.nn.functional.one_hot` requires that.
    """
    ans = {}
    for utt_id, ali in alignments.items():
        ali = torch.tensor(ali, dtype=torch.int64, device=device)
        ans[utt_id] = ali
    return ans


def lookup_alignments(
    cut_ids: List[str],
    alignments: Dict[str, torch.Tensor],
    num_classes: int,
    log_score: float = -10,
) -> torch.Tensor:
    """Return a mask constructed from alignments by a list of cut IDs.

    The returned mask is a 3-D tensor of shape (N, T, C). For each frame,
    i.e., each row, of the returned mask, positions not corresponding to
    the alignments are filled with `log_score`, while the position
    specified by the alignment is filled with 0. For instance, if the alignments
    of two utterances are:

        [ [1, 3, 2], [1, 0, 4, 2] ]
    num_classes is 5 and log_score is -10,  then the returned mask is

        [
          [[-10, 0, -10, -10, -10],
           [-10, -10, -10, 0, -10],
           [-10, -10, 0, -10, -10],
           [0, -10, -10, -10, -10]],
          [[-10, 0, -10, -10, -10],
           [0, -10, -10, -10, -10],
           [-10, -10, -10, -10, 0],
           [-10, -10, 0, -10, -10]]
        ]
    Note: We pad the alignment of the first utterance with 0.

    Args:
      cut_ids:
        A list of utterance IDs.
      alignments:
        A dict containing alignments. The keys are utterance IDs and the values
        are framewise alignments.
      num_classes:
        The max token ID + 1 that appears in the alignments.
      log_score:
        Positions in the returned tensor not corresponding to the alignments
        are filled with this value.
    Returns:
      Return a 3-D torch.float32 tensor of shape (N, T, C).
    """
    # We assume all utterances have their alignments.
    ali = [alignments[cut_id] for cut_id in cut_ids]
    padded_ali = pad_sequence(ali, batch_first=True, padding_value=0)
    padded_one_hot = torch.nn.functional.one_hot(
        padded_ali,
        num_classes=num_classes,
    )
    mask = (1 - padded_one_hot) * float(log_score)
    return mask



import os
import random

from scipy.io import wavfile

from gxl_ai_utils.utils import utils_file
from scipy.signal import fftconvolve
from rir_utils import read_audio, write_wav_with_normal


# 首先得到所有rir音频
def do_get_rir():
    input_wav_dir = "/home/work_nfs8/xlgeng/data/rir_data/simulated_rirs_16k"
    wav_list = utils_file.do_get_list_for_wav_dir(input_wav_dir, recursive=True)
    print(f'len: {len(wav_list)}')
    output_wav_list_path = './result/rir_list_173M.txt'
    utils_file.write_list_to_file(wav_list, output_wav_list_path)


def do_conv_rir_to_one_wav(input_wav_path, output_dir, rir_wav_path):
    """"""
    fs, audio = wavfile.read(input_wav_path)
    output_path = utils_file.do_replace_dir(input_wav_path, output_dir)
    print(output_path)
    # 将RIR应用到音频上
    rir = read_audio(rir_wav_path)
    audio_reverb = fftconvolve(audio, rir)[:len(audio)]
    write_wav_with_normal(output_path, audio_reverb, fs)


def do_conv_rir_to_list_wav(input_wav_list, output_dir, rir_list_path):
    """"""
    rir_path_list = utils_file.load_list_file_clean(rir_list_path)
    rir_num = len(rir_path_list)
    for input_wav_path in utils_file.tqdm(input_wav_list, desc='do_conv_rir_to_list_wav', total=len(input_wav_list)):
        rir_index = random.randint(0, rir_num - 1)
        rir_wav_path = rir_path_list[rir_index]
        fs, audio = wavfile.read(input_wav_path)
        output_path = utils_file.do_replace_dir(input_wav_path, output_dir)
        # 将RIR应用到音频上
        rir = read_audio(rir_wav_path)
        audio_reverb = fftconvolve(audio, rir)[:len(audio)]
        write_wav_with_normal(output_path, audio_reverb, fs)

        # do_conv_rir_to_one_wav(input_wav_path, output_dir, rir_wav_path)


# 构造将rir加入普通wav的脚本
def do_conv_rir_to_wav_main():
    big_name = 'pici_45093H'
    second_name = 'tiqianpi_1_3500H'
    input_wav_scp = f"/home/work_nfs8/xlgeng/aslp_spider_data/{big_name}/{second_name}/wav.scp"
    text_text_scp = f"/home/work_nfs8/xlgeng/aslp_spider_data/{big_name}/{second_name}/text"
    output_wav_store_dir_root_path = "/home/work_nfs10/xlgeng/data/pachong_row_data/rir_wav/"
    output_wav_store_dir = f"{output_wav_store_dir_root_path}/{big_name}/{second_name}"
    output_wav_scp_dir_root_path = "/home/work_nfs8/xlgeng/data/pachong_row_scp/"
    output_wav_scp_dir = f"{output_wav_scp_dir_root_path}/{big_name}/{second_name}"
    utils_file.makedir_sil(output_wav_store_dir)
    utils_file.makedir_sil(output_wav_store_dir)
    input_wav_list = list(utils_file.load_dict_from_scp(input_wav_scp).values())
    input_wav_list_list = utils_file.do_split_list(input_wav_list, 20)
    runner = utils_file.GxlDynamicProcessPool()
    rir_list_path = './result/rir_list_173M.txt'
    for i, input_wav_list in enumerate(input_wav_list_list):
        output_wav_store_dir_i = os.path.join(output_wav_store_dir, f"split_{i}")
        utils_file.makedir_sil(output_wav_store_dir_i)
        runner.add_task(do_conv_rir_to_list_wav, [input_wav_list, output_wav_store_dir_i, rir_list_path])
    runner.run()
    utils_file.do_get_scp_for_wav_dir(output_wav_store_dir, wav_scp_file_path=os.path.join(output_wav_scp_dir, 'wav.scp'), recursive=True)
    utils_file.copy_file(source_path=text_text_scp, destination_path=os.path.join(output_wav_scp_dir, 'text'), use_shell=True)


if __name__ == '__main__':
    do_conv_rir_to_wav_main()



import torch
from torch import nn


class Joiner(nn.Module):
    def __init__(self, input_size, output_size):
        super(Joiner, self).__init__()
        self.input_size = input_size
        self.output_linear = nn.Linear(input_size, output_size)

    def forward(self, encoder_out, decoder_out):
        assert encoder_out.ndim == decoder_out.ndim == 3, f"encoder out dim: {encoder_out.ndim}, decoder out dim: {decoder_out.ndim}"
        assert encoder_out.size(0) == decoder_out.size(0), f"encoder_out.size(0): {encoder_out.size(0)}, decoder_out.size(0): {decoder_out.size(0)}"
        assert encoder_out.size(2) == decoder_out.size(2), f"encoder_out.size(2): {encoder_out.size(2)}, decoder_out.size(2): {decoder_out.size(2)}"
        encoder_out = encoder_out.unsqueeze(2)
        decoder_out = decoder_out.unsqueeze(1)
        logits = encoder_out + decoder_out
        logits = nn.functional.relu(logits)
        output = self.output_linear(logits)
        return output


def test_joiner():
    model = Joiner(80, 1024)
    input_encoder = torch.randn(3, 93, 80)
    input_decoder = torch.randn(3, 12, 80)
    output = model(input_encoder, input_decoder)
    print(output.size())



import os.path
import re
import time

from tqdm import tqdm

from gxl_ai_utils.utils import utils_spider, utils_file
from bs4 import BeautifulSoup


# ai_yuyin_zhuanlan_url = "https://www.zhihu.com/column/c_1306669801105391616"
# article_url_xpath = "/html/body/div[1]/div/main/div/div[4]/div/div/div/div/h2/span/a/@href"

def get_zhuanji_list_by_search_word(search_word="语音"):
    """"""
    # driver = webdriver.Chrome()
    # driver.implicitly_wait(50)
    # time.sleep(50)
    url = f'https://www.zhihu.com/search?q={search_word}&type=column'
    url = "https://www.zhihu.com/search?type=column&q=%E5%93%88%E5%93%88"
    # page_str = utils_spider.send_request(url).text
    # print(page_str)
    with open("./zhihu_search_by_语音.html", 'r', encoding='utf-8') as file:
        page_str = file.read()
    print(page_str)
    # page_str, driver = utils_spider.get_source_page_from_url(url, is_debug_chrome=False, wait_time=5)
    # href_xpath =  "/html/body/div[1]/div/main/div/div[2]/div[3]/div/div/div/div/div/div/div/div/div[2]/h2/span/div/a/@href"
    href_xpath = '//*[@id="SearchMain"]/div/div/div/div/div/div/div/div/div[2]/h2/span/div/a/@href'
    href_list = utils_spider.handle_xpath(page_str, href_xpath)
    res_dict = []
    for href in href_list:
        print(href)
        res_dict.append(href)
    utils_file.write_list_to_file(res_dict, "./output/zhihu/zhuanji_list.txt")

def do_get_url_for_a_zhaunji(zhuanji_url:str):
    ai_yuyin_zhuanlan_url = zhuanji_url
    zhuanji_name_xpath = "/html/body/div[1]/div/main/div/div[2]/div/div[1]/text()"
    article_url_xpath = "/html/body/div[1]/div/main/div/div[4]/div/div/div/div/h2/span/a/@href"
    output_dir = './output/zhihu'
    utils_file.makedir_sil(output_dir)
    page_str, driver = utils_spider.get_source_page_from_url(ai_yuyin_zhuanlan_url)
    article_path_list = utils_spider.handle_xpath(page_str, article_url_xpath)
    zhuanji_name = utils_spider.handle_xpath(page_str, zhuanji_name_xpath)[0]
    now_lens = len(article_path_list)
    for i in range(500):
        # 执行JavaScript来滚动滑轮
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        # 等待页面加载
        time.sleep(2)
        page_str = driver.page_source
        article_path_list = utils_spider.handle_xpath(page_str, article_url_xpath)
        next_lens = len(article_path_list)
        if now_lens == next_lens:
            break
        else:
            now_lens = next_lens
        utils_file.logging_print(f"i:{i} now_lens:{now_lens} next_lens:{next_lens}")
    utils_file.write_list_to_file(article_path_list,
                                  os.path.join(output_dir, "article_path_list", f"article_path_list_{zhuanji_name}.txt"))

    output_file_path = os.path.join(output_dir, "article_content", f"zhihu_{zhuanji_name}.jsonl")
    utils_file.makedir_for_file_or_dir(output_file_path)
    for url in tqdm(article_path_list, total=len(article_path_list)):
        url = 'https:' + url
        dict_res = handle_a_page_url(url, driver)
        utils_file.write_single_dict_to_jsonl(dict_res, output_file_path)


def handle_a_page_url(page_url, driver):
    """
    知乎的request得到的文本被加密了,无法解码成正常字符串,都是乱码
    :param page_url:
    :return:
    """
    # response = utils_spider.send_request(page_url, encoding='utf-8')
    # content = response.content
    # print(response.apparent_encoding)
    # print(response.encoding)
    # html_str = content[100:].decode()
    # soup = BeautifulSoup(response.content, "html.parser")
    # 打印解析后的文本内容
    # html_str = (soup.get_text())
    id = page_url.split("/")[-1]
    html_str, driver = utils_spider.get_source_page_from_url_have_driver(page_url, driver)
    title_xpath = "/html/body/div[1]/div/main/div/article/header/h1/text()"
    title = utils_spider.handle_xpath(html_str, title_xpath)[0]
    title = title.replace("\u200b", "")
    title = title.replace("​", "")
    content_xpath = "/html/body/div[1]/div/main/div/article/div[1]/div/div/div//text()"
    res = utils_spider.handle_xpath(html_str, content_xpath)
    # print(res)
    final_txt = ""
    for txt in res:
        if txt is None or txt == "" or txt == "\n" or txt == "\t" or txt == "\u200b":
            continue
        final_txt += txt
    final_txt = final_txt.replace("\u200b", "")
    final_txt = re.sub(r'\s{2,}', ' ', final_txt)
    final_txt = final_txt.replace("\n", "")
    return dict(id=id, title=title, content=final_txt)


if __name__ == '__main__':
    """"""
    all_zhuanji_list = utils_file.load_list_file_clean("./output/zhihu/zhuanji_list.txt")
    runner = utils_file.GxlFixedThreadPool(10)
    for zhuanji_url in all_zhuanji_list:
        zhuanji_url = 'https:' + zhuanji_url
        runner.add_task(do_get_url_for_a_zhaunji, [zhuanji_url])
    runner.start()



"""
将所有数据搬运到转移盘
"""
import os

import tqdm

from gxl_ai_utils.utils import utils_file

output_dir = "/home/asr_transfer_dir"
data_conf_path = "./data_multi_task_3.yaml"
data_dict = utils_file.load_dict_from_yaml(data_conf_path)
for key, value in data_dict.items():
    utils_file.logging_info('开始处理如下数据：', key)
    shards_path = value["path"]
    shards_list = utils_file.load_list_file_clean(shards_path)
    output_dir_tmp = os.path.join(output_dir, key)
    utils_file.makedir_sil(output_dir_tmp)
    for shard_item_path in tqdm.tqdm(shards_list, desc=f"copy shards:{key}", total=len(shards_list)):
        output_file_path_item = utils_file.join_path(output_dir_tmp, os.path.basename(shard_item_path))
        utils_file.copy_file(shard_item_path, output_file_path_item, use_shell=True, is_jump=True)
    utils_file.logging_info('处理完成',key)
    utils_file.do_get_list_for_wav_dir(output_dir_tmp,os.path.join(output_dir_tmp,'shards_list.txt'),
                                       suffix='tar')



from gxl_ai_utils.utils import utils_file

input_token_data_dir = "/mnt/sfs/asr/update_data/3500_chat"
old_path = f"{input_token_data_dir}/shards_list_old.txt"
input_path = f"{input_token_data_dir}/shards_list.txt"
path_list = utils_file.load_list_file_clean(input_path)
res_list = []
for path_i in path_list:
    if not utils_file.if_file_exist(path_i) or utils_file.get_file_size(path_i)< 1:
        print(f'{path_i} not exist or size is 0')
    else:
        res_list.append(path_i)

utils_file.copy_file(input_path, old_path)
utils_file.write_list_to_file(res_list, input_path)



# Please install OpenAI SDK first: `pip3 install openai`
import json

from openai import OpenAI

client = OpenAI(api_key="sk-d03313dd7c3a44a281187749c4203dda", base_url="https://api.deepseek.com")
print("Deepseek API client created successfully.")
# prompt = '我希望构造$询问输入语音的说话人的情感信息$的句子，具体情况是一个人为另一个人播放了一段音频，并询问音频中说话人的情感。我将为你提供一些具体的限制，你来为我生成多样的句子。句式:祈使句。 情感一词的表述： "情愫",具体场景为:$"写字楼茶水间"$, 代词为"她"（音频中的说话人的代词），询问两者的关系为: $"邻居关系"$，音频的同义词为：" "录音",说话人的同义词："谈话者". 请结合对话发生的场景和双方关系开始构造对音频中说话人的情感的询问的句子。直接返回构造的句子，以json list的形式返回'
# prompt = '我希望构造$询问输入语音的说话人的情感信息$的句子，具体情况是一个人为另一个人播放了一段音频，并询问音频中说话人的情感。我将为你提供一些具体的限制，你来为我生成多样的句子。句式:祈使句 。 情感一词的表述： "情愫",具体场景为:"写字楼茶水间", 代词为：不使用（音频中的说话人的代词），询问两者的关系为: "邻居关系"，音频的同义词为：" "录音",说话人的同义词："谈话者"。 我希望你先根据关系和场景构造核实的上文背景并直接输出，然后再构造符合改场景的句子，并以json list的形式只返回构造的句子（不包括上文）。'
prompt = '我希望构造$询问输入语音的说话人的情感信息$的句子，具体情况是一个人为另一个人播放了一段音频，并询问音频中说话人的情感。我将为你提供一些具体的限制，你来为我生成多样的句子。句式:询问句 。 情感一词的表述： "情愫", 代词为"她"（音频中的说话人的代词），音频的同义词为：" "录音",说话人的同义词："谈话者".请结合上述内容造对音频中说话人的情感的询问的句子。直接返回构造的句子，以json list的形式返回'
response = client.chat.completions.create(
    model="deepseek-chat",
    temperature=1.5,
    messages=[
        {"role": "system", "content": "You are a helpful assistant"},
        {"role": "user", "content": f"{prompt}"},
    ],
    stream=False
)
res = response.choices[0].message.content
print(res)
start_index = res.find("[")
end_index = res.find("]")+1
res = res[start_index:end_index]
str_list = json.loads(res)
print(str_list)
res_file = 'res.txt'
with open(res_file, 'a', encoding='utf8') as f:
    for s in str_list:
        f.write(s+'\n')


import torch
import torch.nn as nn
import time

class MyModel(nn.Module):
    def __init__(self):
        super(MyModel, self).__init__()
        self.linears = nn.Sequential(
            nn.Linear(100, 20000),
            nn.Linear(20000, 2000),
            nn.Linear(2000, 10000),
        )

    def forward(self, data):
        return self.linears(data)

data_input = [
    torch.randn(43, 100) for _ in range(10000)
]
model = MyModel()
optimer = torch.optim.Adam(model.parameters(), lr=0.001)
model = model.to('cuda:0')
while True:
    time.sleep(10)



import copy
import logging
from typing import Optional, Tuple, Union

import humanfriendly
import torch
import os

from s3prl.upstream.hubert.convert import load_and_convert_fairseq_ckpt

os.environ["CUDA_VISIBLE_DEVICES"] = "6"



class S3prlFrontend(torch.nn.Module):
    """Speech Pretrained Representation frontend structure for ASR."""

    def __init__(
        self,
        fs: Union[int, str] = 16000,
        frontend_conf: Optional[dict] = None,
        download_dir: str = None,
        multilayer_feature: bool = False,
        layer: int = -1,
    ):
        try:
            import s3prl
            from s3prl.nn import Featurizer, S3PRLUpstream
            from s3prl.upstream.hubert.convert import load_and_convert_fairseq_ckpt
        except Exception as e:
            print("Error: S3PRL is not properly installed.")
            print("Please install S3PRL: cd ${MAIN_ROOT}/tools && make s3prl.done")
            raise e

        super().__init__()

        if isinstance(fs, str):
            fs = humanfriendly.parse_size(fs)
        if fs != 16000:
            logging.warning(
                "All the upstream models in S3PRL now only support 16 kHz audio."
            )

        if download_dir is not None:
            s3prl.util.download.set_dir(download_dir)

        assert frontend_conf.get("upstream", None) in S3PRLUpstream.available_names()
        upstream = S3PRLUpstream(
            frontend_conf.get("upstream"),
            path_or_url=frontend_conf.get("upstream_ckpt", None),
            normalize=frontend_conf.get("normalize", False),
            extra_conf=frontend_conf.get("extra_conf", None),
        )
        # upstream = hubert_local(
        #     ckpt=frontend_conf.get("upstream_ckpt"),
        #     normalize=frontend_conf.get("normalize", False),
        # )

        if getattr(upstream.upstream, "model", None):
            if getattr(upstream.upstream.model, "feature_grad_mult", None) is not None:
                upstream.upstream.model.feature_grad_mult = 1.0
        upstream.eval()

        if layer != -1:
            layer_selections = [layer]
            assert (
                not multilayer_feature
            ), "multilayer feature will be deactivated, when specific layer used"
        else:
            layer_selections = None
        featurizer = Featurizer(upstream, layer_selections=layer_selections)

        self.multilayer_feature = multilayer_feature
        self.layer = layer
        self.upstream, self.featurizer = upstream, featurizer
        self.pretrained_params = copy.deepcopy(self.upstream.state_dict())
        self.frontend_type = "s3prl"
        self.hop_length = self.featurizer.downsample_rate
        self.tile_factor = frontend_conf.get("tile_factor", 1)

    def _tile_representations(self, feature):
        """Tile up the representations by `tile_factor`.

        Input - sequence of representations
                shape: (batch_size, seq_len, feature_dim)

        Output - sequence of tiled representations
                 shape: (batch_size, seq_len * factor, feature_dim)
        """
        assert (
            len(feature.shape) == 3
        ), "Input argument `feature` has invalid shape: {}".format(feature.shape)
        tiled_feature = feature.repeat(1, 1, self.tile_factor)
        tiled_feature = tiled_feature.reshape(
            feature.size(0), feature.size(1) * self.tile_factor, feature.size(2)
        )
        return tiled_feature

    def output_size(self) -> int:
        return self.featurizer.output_size

    def forward(
        self, input: torch.Tensor, input_lengths: torch.Tensor
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        feats, feats_lens = self.upstream(input, input_lengths)
        if self.layer != -1:
            layer = self.layer
            feats, feats_lens = feats[layer], feats_lens[layer]
            return feats, feats_lens

        if self.multilayer_feature:
            feats, feats_lens = self.featurizer(feats, feats_lens)
        else:
            feats, feats_lens = self.featurizer(feats[-1:], feats_lens[-1:])

        if self.tile_factor != 1:
            feats = self._tile_representations(feats)

        return feats, feats_lens

    def reload_pretrained_parameters(self):
        self.upstream.load_state_dict(self.pretrained_params)
        logging.info("Pretrained S3PRL frontend model parameters reloaded!")


if __name__ == "__main__":
    source_fairseq_path = "/home/work_nfs15/asr_data/ckpt/origin_chinese_hubert/chinese_hubert_large.pt"
    s3qrl_path = "/home/work_nfs15/asr_data/ckpt/origin_chinese_hubert/chinese_hubert_large_s3qrl.pt"
    # load_and_convert_fairseq_ckpt(source_fairseq_path, s3qrl_path) # 成功转换
    hubert_model = S3prlFrontend(
        frontend_conf={
            "upstream": "hubert_local",
            "upstream_model_config":"",
            "upstream_ckpt": s3qrl_path,
        },
        download_dir="./hub",
    multilayer_feature=True,)
    print(hubert_model)
    input_wav_tensor = torch.randn(5, 16000*10)
    # 随机出5个长度
    input_wav_lens = torch.randint(16000*5, 16000*10, (5,))
    feats, feats_lens = hubert_model(input_wav_tensor, input_wav_lens)
    print(feats.shape, feats_lens.shape)



import logging
from pathlib import Path
from typing import Iterable, List, Tuple, Union

import k2
import torch

from icefall.lexicon import UniqLexicon


class MmiTrainingGraphCompiler(object):
    def __init__(
        self,
        lang_dir: Path,
        uniq_filename: str = "uniq_lexicon.txt",
        device: Union[str, torch.device] = "cpu",
        oov: str = "<UNK>",
        sos_id: int = 1,
        eos_id: int = 1,
    ):
        """
        Args:
          lang_dir:
            Path to the lang directory. It is expected to contain the
            following files::

                - tokens.txt
                - words.txt
                - P.fst.txt

            The above files are generated by the script `prepare.sh`. You
            should have run it before running the training code.
          uniq_filename:
            File name to the lexicon in which every word has exactly one
            pronunciation. We assume this file is inside the given `lang_dir`.

          device:
            It indicates CPU or CUDA.
          oov:
            Out of vocabulary word. When a word in the transcript
            does not exist in the lexicon, it is replaced with `oov`.
        """
        self.lang_dir = Path(lang_dir)
        self.lexicon = UniqLexicon(lang_dir, uniq_filename=uniq_filename)
        self.device = torch.device(device)

        self.L_inv = self.lexicon.L_inv.to(self.device)

        self.oov_id = self.lexicon.word_table[oov]
        self.sos_id = sos_id
        self.eos_id = eos_id

        self.build_ctc_topo_P()

    def build_ctc_topo_P(self):
        """Built ctc_topo_P, the composition result of
        ctc_topo and P, where P is a pre-trained bigram
        word piece LM.
        """
        # Note: there is no need to save a pre-compiled P and ctc_topo
        # as it is very fast to generate them.
        logging.info(f"Loading P from {self.lang_dir/'P.fst.txt'}")
        with open(self.lang_dir / "P.fst.txt") as f:
            # P is not an acceptor because there is
            # a back-off state, whose incoming arcs
            # have label #0 and aux_label 0 (i.e., <eps>).
            P = k2.Fsa.from_openfst(f.read(), acceptor=False)

        first_token_disambig_id = self.lexicon.token_table["#0"]

        # P.aux_labels is not needed in later computations, so
        # remove it here.
        del P.aux_labels
        # CAUTION: The following line is crucial.
        # Arcs entering the back-off state have label equal to #0.
        # We have to change it to 0 here.
        labels = P.labels.clone()
        labels[labels >= first_token_disambig_id] = 0
        P.labels = labels

        P = k2.remove_epsilon(P)
        P = k2.arc_sort(P)
        P = P.to(self.device)
        # Add epsilon self-loops to P because we want the
        # following operation "k2.intersect" to run on GPU.
        P_with_self_loops = k2.add_epsilon_self_loops(P)

        max_token_id = max(self.lexicon.tokens)
        logging.info(
            f"Building ctc_topo (modified=False). max_token_id: {max_token_id}"
        )
        ctc_topo = k2.ctc_topo(max_token_id, modified=False, device=self.device)

        ctc_topo_inv = k2.arc_sort(ctc_topo.invert_())

        logging.info("Building ctc_topo_P")
        ctc_topo_P = k2.intersect(
            ctc_topo_inv, P_with_self_loops, treat_epsilons_specially=False
        ).invert()

        self.ctc_topo_P = k2.arc_sort(ctc_topo_P)
        logging.info(f"ctc_topo_P num_arcs: {self.ctc_topo_P.num_arcs}")

    def compile(
        self, texts: Iterable[str], replicate_den: bool = True
    ) -> Tuple[k2.Fsa, k2.Fsa]:
        """Create numerator and denominator graphs from transcripts
        and the bigram phone LM.

        Args:
          texts:
            A list of transcripts. Within a transcript, words are
            separated by spaces. An example `texts` is given below::

                ["Hello icefall", "LF-MMI training with icefall using k2"]

          replicate_den:
            If True, the returned den_graph is replicated to match the number
            of FSAs in the returned num_graph; if False, the returned den_graph
            contains only a single FSA
        Returns:
          A tuple (num_graph, den_graph), where

            - `num_graph` is the numerator graph. It is an FsaVec with
              shape `(len(texts), None, None)`.

            - `den_graph` is the denominator graph. It is an FsaVec
              with the same shape of the `num_graph` if replicate_den is
              True; otherwise, it is an FsaVec containing only a single FSA.
        """
        transcript_fsa = self.build_transcript_fsa(texts)

        # remove word IDs from transcript_fsa since it is not needed
        del transcript_fsa.aux_labels
        # NOTE: You can comment out the above statement
        # if you want to run test/test_mmi_graph_compiler.py

        transcript_fsa_with_self_loops = k2.remove_epsilon_and_add_self_loops(
            transcript_fsa
        )

        transcript_fsa_with_self_loops = k2.arc_sort(transcript_fsa_with_self_loops)

        num = k2.compose(
            self.ctc_topo_P,
            transcript_fsa_with_self_loops,
            treat_epsilons_specially=False,
        )

        # CAUTION: Due to the presence of P,
        # the resulting `num` may not be connected
        num = k2.connect(num)

        num = k2.arc_sort(num)

        ctc_topo_P_vec = k2.create_fsa_vec([self.ctc_topo_P])
        if replicate_den:
            indexes = torch.zeros(len(texts), dtype=torch.int32, device=self.device)
            den = k2.index_fsa(ctc_topo_P_vec, indexes)
        else:
            den = ctc_topo_P_vec

        return num, den

    def build_transcript_fsa(self, texts: List[str]) -> k2.Fsa:
        """Convert transcripts to an FsaVec with the help of a lexicon
        and word symbol table.

        Args:
          texts:
            Each element is a transcript containing words separated by space(s).
            For instance, it may be 'HELLO icefall', which contains
            two words.

        Returns:
          Return an FST (FsaVec) corresponding to the transcript.
          Its `labels` is token IDs and `aux_labels` is word IDs.
        """
        word_ids_list = []
        for text in texts:
            word_ids = []
            for word in text.split():
                if word in self.lexicon.word_table:
                    word_ids.append(self.lexicon.word_table[word])
                else:
                    word_ids.append(self.oov_id)
            word_ids_list.append(word_ids)

        fsa = k2.linear_fsa(word_ids_list, self.device)
        fsa = k2.add_epsilon_self_loops(fsa)

        # The reason to use `invert_()` at the end is as follows:
        #
        # (1) The `labels` of L_inv is word IDs and `aux_labels` is token IDs
        # (2) `fsa.labels` is word IDs
        # (3) after intersection, the `labels` is still word IDs
        # (4) after `invert_()`, the `labels` is token IDs
        #     and `aux_labels` is word IDs
        transcript_fsa = k2.intersect(
            self.L_inv, fsa, treat_epsilons_specially=False
        ).invert_()
        transcript_fsa = k2.arc_sort(transcript_fsa)
        return transcript_fsa

    def texts_to_ids(self, texts: List[str]) -> List[List[int]]:
        """Convert a list of texts to a list-of-list of piece IDs.

        Args:
          texts:
            It is a list of strings. Each string consists of space(s)
            separated words. An example containing two strings is given below:

                ['HELLO ICEFALL', 'HELLO k2']
            We assume it contains no OOVs. Otherwise, it will raise an
            exception.
        Returns:
          Return a list-of-list of token IDs.
        """
        return self.lexicon.texts_to_token_ids(texts).tolist()



# Copyright    2021  Xiaomi Corp.        (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Tuple

import torch
import torch.nn as nn


class EncoderInterface(nn.Module):
    def forward(
        self, x: torch.Tensor, x_lens: torch.Tensor
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        """
        Args:
          x:
            A tensor of shape (batch_size, input_seq_len, num_features)
            containing the input features.
          x_lens:
            A tensor of shape (batch_size,) containing the number of frames
            in `x` before padding.
        Returns:
          Return a tuple containing two tensors:
            - encoder_out, a tensor of (batch_size, out_seq_len, output_dim)
              containing unnormalized probabilities, i.e., the output of a
              linear layer.
            - encoder_out_lens, a tensor of shape (batch_size,) containing
              the number of frames in `encoder_out` before padding.
        """
        raise NotImplementedError("Please implement it in a subclass")



# Copyright (c)  2022  Xiaomi Corporation (authors: Xiaoyu Yang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Optional, Tuple

import torch
import torch.nn.functional as F

from icefall.transformer_lm.encoder import Transformer
from icefall.utils import AttributeDict, add_eos, add_sos, make_pad_mask


class TransformerLM(torch.nn.Module):
    def __init__(
        self,
        vocab_size: int,
        embedding_dim: int,
        d_model: int,
        dim_feedforward: int,
        nhead: int = 8,
        num_layers: int = 16,
        tie_weights: bool = True,
        dropout: float = 0.1,
        emb_dropout_rate: float = 0.0,
        params: AttributeDict = None,
    ):
        super().__init__()

        self.vocab_size = vocab_size
        self.params = params

        self.input_embedding = torch.nn.Embedding(
            num_embeddings=vocab_size,
            embedding_dim=embedding_dim,
        )

        self.encoder = Transformer(
            input_dim=embedding_dim,
            d_model=d_model,
            dim_feedforward=dim_feedforward,
            nhead=nhead,
            num_layers=num_layers,
            dropout_rate=dropout,
        )

        self.output_linear = torch.nn.Linear(
            in_features=d_model, out_features=vocab_size
        )
        if tie_weights:
            logging.info("Tying weights")
            assert d_model == embedding_dim, (d_model, embedding_dim)
            self.output_linear.weight = self.input_embedding.weight
        else:
            logging.info("Not tying weights")

    def forward(
        self,
        x: torch.Tensor,
        y: torch.Tensor,
        x_lens: torch.Tensor,
        return_logits: bool = False,
    ):
        """Forward transformer language model

        Args:
            x (torch.Tensor): Input tokens (B,L)
            y (torch.Tensor): Output tokens (with EOS appended) (B,L)
            x_lens (torch.Tensor): Length of input tokens before padding (B,)
            return_logits (bool, optional): Return logits instead of NLL

        """

        x = self.input_embedding(x)

        x, x_lens = self.encoder(x, x_lens)

        logits = self.output_linear(x)

        if return_logits:
            return logits

        nll_loss = F.cross_entropy(
            logits.reshape(-1, self.vocab_size), y.reshape(-1), reduction="none"
        )

        mask = make_pad_mask(x_lens).reshape(-1)
        nll_loss.masked_fill_(mask, 0)

        return nll_loss

    def score_token(self, x: torch.Tensor, x_lens: torch.Tensor, state=None):
        bs = x.size(0)

        state = None
        logits = self.forward(x, x, x_lens, return_logits=True)
        index = torch.arange(bs)

        last_logits = logits[index, x_lens - 1, :]

        return last_logits.log_softmax(-1), state



import glob
import os.path
import random

from gxl_ai_utils.utils import utils_file

def data_handle():
    """"""
    input_raw_dir = "/home/node36_data/xlgeng/asr_data_from_pachong/gxl_output/lizhi_jiankang"
    text_with_puncuation = os.path.join(input_raw_dir, "text_with_punctuation")
    wav_path = os.path.join(input_raw_dir, "wav.scp")
    output_dir = "/home/work_nfs14/xlgeng/asr_data_shard/pachong_data/4_lizhi_jiankang_punctuation"
    os.makedirs(output_dir, exist_ok=True)
    utils_file.do_make_shard_file(wav_path, text_with_puncuation, output_dir)
    utils_file.copy_file(os.path.join(output_dir, "shards_list.txt"),"/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/litle_data_with_puncuation_list.list")


def do_get_random_sublist(input_list, num):
    """"""
    return [input_list[i] for i in sorted(random.sample(range(len(input_list)), num))]
def data_prepare_2():
    """"""
    aishell_dir = "/home/local_data/hwang/huawei_cn_en/cn/aishell1/"
    aishell_list = glob.glob(os.path.join(aishell_dir, "*.tar"))
    punctuation_path = utils_file.load_list_file_clean("/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/litle_data_with_puncuation_list.list")
    aishell_list_little = do_get_random_sublist(aishell_list, 100)
    punctuation_list_little = do_get_random_sublist(punctuation_path, 100)
    output_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/mix_tast.list"
    res_list = []
    for aishell_list_little_i in aishell_list_little:
        temp_i = f'{aishell_list_little_i}\t0'
        res_list.append(temp_i)
    for punctuation_list_little_i in punctuation_list_little:
        temp_i = f'{punctuation_list_little_i}\t1'
        res_list.append(temp_i)
    random.shuffle(res_list)
    utils_file.write_list_to_file(res_list, output_path)

def kespeech():
    input_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/test"
    output_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/test_little"
    utils_file.copy_family_file(input_dir, output_dir)

def mix_train_data_xinan():
    """"""
    common_data_list_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/gxl_all_new_wenetspeech.list"
    xinan_data_wav_path= "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/train/wav.scp"
    xinan_data_text_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/train/text"
    xinan_data_shard_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/train/shard"
    utils_file.makedir_sil(xinan_data_shard_dir)
    utils_file.do_make_shard_file(xinan_data_wav_path, xinan_data_text_path, xinan_data_shard_dir)
    xinan_data_list_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/kespeech/Southwestern/train/shard/shards_list.txt"
    xinan_list = utils_file.load_list_file_clean(xinan_data_list_path)
    common_list = utils_file.load_list_file_clean(common_data_list_path)
    little_common_list = utils_file.do_get_random_sublist(common_list, 150)
    output_path = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/data_list/mix_train_data_xinan.list"
    res_list = []
    for line_i in little_common_list:
        line_temp = f'{line_i} 2'
        res_list.append(line_temp)
    for line_i in xinan_list:
        line_temp = f'{line_i} 4'
        res_list.append(line_temp)
    random.shuffle(res_list)
    utils_file.write_list_to_file(res_list, output_path)


if  __name__ == "__main__":
    """"""
    mix_train_data_xinan()



#!/usr/bin/env python3
# Copyright    2021  Xiaomi Corp.        (authors: Fangjun Kuang
#                                                  Xiaoyu Yang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import math
from pathlib import Path

import torch
from dataset import get_dataloader
from train import get_params

from icefall.checkpoint import average_checkpoints, load_checkpoint
from icefall.transformer_lm.model import TransformerLM
from icefall.utils import AttributeDict, setup_logger, str2bool


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--epoch",
        type=int,
        default=7,
        help="It specifies the checkpoint to use for decoding."
        "Note: Epoch counts from 0.",
    )
    parser.add_argument(
        "--avg",
        type=int,
        default=1,
        help="Number of checkpoints to average. Automatically select "
        "consecutive checkpoints before the checkpoint specified by "
        "'--epoch'. ",
    )

    parser.add_argument(
        "--exp-dir",
        type=str,
        default="transformer_lm/exp_full_libri_16layer_maxlen200_8gpu",
    )

    parser.add_argument(
        "--lm-data",
        type=str,
        help="Path to the LM test data for computing perplexity",
        default="transformer_lm/libri_lm_training_bpe500/sorted_lm_data-test.pt",
    )

    parser.add_argument(
        "--vocab-size",
        type=int,
        default=500,
        help="Vocabulary size of the model",
    )

    parser.add_argument(
        "--num-layers",
        type=int,
        default=16,
        help="Number of RNN layers the model",
    )

    parser.add_argument(
        "--tie-weights",
        type=str2bool,
        default=False,
        help="""True to share the weights between the input embedding layer and the
        last output linear layer
        """,
    )

    parser.add_argument(
        "--batch-size",
        type=int,
        default=50,
        help="Number of RNN layers the model",
    )

    parser.add_argument(
        "--max-sent-len",
        type=int,
        default=100,
        help="Number of RNN layers the model",
    )

    return parser


def main():
    parser = get_parser()
    args = parser.parse_args()
    args.exp_dir = Path(args.exp_dir)
    args.lm_data = Path(args.lm_data)

    params = get_params()
    params.update(vars(args))

    setup_logger(f"{params.exp_dir}/log-ppl/")
    logging.info("Computing perplexity started")
    logging.info(params)

    device = torch.device("cpu")
    if torch.cuda.is_available():
        device = torch.device("cuda", 0)

    logging.info(f"Device: {device}")

    logging.info("About to create model")
    model = TransformerLM(
        vocab_size=params.vocab_size,
        d_model=params.encoder_dim,
        embedding_dim=params.embedding_dim,
        dim_feedforward=params.dim_feedforward,
        nhead=params.nhead,
        num_layers=params.num_layers,
        tie_weights=params.tie_weights,
        params=params,
    )

    if params.avg == 1:
        load_checkpoint(f"{params.exp_dir}/epoch-{params.epoch}.pt", model)
        model.to(device)
    else:
        start = params.epoch - params.avg + 1
        filenames = []
        for i in range(start, params.epoch + 1):
            if start >= 0:
                filenames.append(f"{params.exp_dir}/epoch-{i}.pt")
        logging.info(f"averaging {filenames}")
        model.to(device)
        model.load_state_dict(average_checkpoints(filenames, device=device))

    model.eval()
    num_param = sum([p.numel() for p in model.parameters()])
    num_param_requires_grad = sum(
        [p.numel() for p in model.parameters() if p.requires_grad]
    )

    logging.info(f"Number of model parameters: {num_param}")
    logging.info(
        f"Number of model parameters (requires_grad): "
        f"{num_param_requires_grad} "
        f"({num_param_requires_grad/num_param_requires_grad*100}%)"
    )

    logging.info(f"Loading LM test data from {params.lm_data}")
    test_dl = get_dataloader(
        filename=params.lm_data,
        is_distributed=False,
        params=params,
    )

    tot_loss = 0.0
    num_tokens = 0
    num_sentences = 0
    for batch_idx, batch in enumerate(test_dl):
        x, y, sentence_lengths = batch
        x = x.to(device)
        y = y.to(device)
        sentence_lengths = sentence_lengths.to(device)

        nll = model(x, y, sentence_lengths)
        loss = nll.sum().cpu().item()

        tot_loss += loss
        num_tokens += sentence_lengths.sum().cpu().item()
        num_sentences += x.size(0)

    ppl = math.exp(tot_loss / num_tokens)
    logging.info(
        f"total nll: {tot_loss}, num tokens: {num_tokens}, "
        f"num sentences: {num_sentences}, ppl: {ppl:.3f}"
    )


if __name__ == "__main__":
    main()



# Copyright      2023  Xiaomi Corp.        (authors: Fangjun Kuang)

import kaldifst


# Note the name contains `standard`; it means there will be non-standard
# topologies.
def build_standard_ctc_topo(max_token_id: int) -> kaldifst.StdVectorFst:
    """Build a standard CTC topology.

    Args:
      Maximum valid token ID. We assume token IDs are contiguous
      and starts from 0. In other words, the vocabulary size is
      ``max_token_id + 1``. We assume the ID of the blank symbol is 0.
    """
    # Token ID starts from 0 and there are as many states as the
    # number of tokens.
    #
    # Note that epsilon is not a token and the token with ID 0 in tokens.txt
    # is not an epsilon. It means input label 0 of the resulting FST does
    # not represent an epsilon.
    #
    # You can use the function `add_one()` to modify the input/output labels
    # of the resulting FST

    num_states = max_token_id + 1

    # Step 1: Create as many states as the number of tokens.
    # Each state is a final state
    fst = kaldifst.StdVectorFst()
    for i in range(num_states):
        s = fst.add_state()
        fst.set_final(state=s, weight=0)

    # Step 2: Set state 0 as the start state.
    # We assume the ID of the blank symbol is 0.
    fst.start = 0

    # Step 3: Build a fully connected graph.
    for i in range(num_states):
        for k in range(num_states):
            fst.add_arc(
                state=i,
                arc=kaldifst.StdArc(
                    ilabel=k,
                    olabel=k if i != k else 0,  # if i==k, it is a self loop
                    weight=0,
                    nextstate=k,
                ),
            )
    # Please see ./test_ctc_topo.py if you want to know what the resulting
    # FST looks like

    return fst


def add_one(
    fst: kaldifst.StdVectorFst,
    treat_ilabel_zero_specially: bool,
    update_olabel: bool,
) -> None:
    """Modify the input and output labels of the given FST in-place.

    Args:
      fst:
        The FST to be modified. It is changed in-place.
      treat_ilabel_zero_specially:
        If True, then every non-zero input label is increased by one and the
        zero input label is not changed.
        If False, then every input label is increased by one.
      update_olabel:
        If False, the output label is not changed.
        If True, then every non-zero output label is increased by one.
        In either case, output label with 0 is not changed.
    """
    for state in kaldifst.StateIterator(fst):
        for arc in kaldifst.ArcIterator(fst, state):
            # If treat_ilabel_zero_specially is False, we always change it
            # Otherwise, we only change non-zero input labels
            if treat_ilabel_zero_specially is False or arc.ilabel != 0:
                arc.ilabel += 1

            if update_olabel and arc.olabel != 0:
                arc.olabel += 1

    if fst.input_symbols is not None:
        input_symbols = kaldifst.SymbolTable()
        input_symbols.add_symbol(symbol="<eps>", key=0)

        for i in range(0, fst.input_symbols.num_symbols()):
            s = fst.input_symbols.find(i)
            input_symbols.add_symbol(symbol=s, key=i + 1)

        fst.input_symbols = input_symbols

    if update_olabel and fst.output_symbols is not None:
        output_symbols = kaldifst.SymbolTable()
        output_symbols.add_symbol(symbol="<eps>", key=0)

        for i in range(0, fst.output_symbols.num_symbols()):
            s = fst.output_symbols.find(i)
            output_symbols.add_symbol(symbol=s, key=i + 1)

        fst.output_symbols = output_symbols


def add_disambig_self_loops(fst: kaldifst.StdVectorFst, start: int, end: int):
    """Add self-loops to each state.

    For each disambig symbol, we add a self-loop with input label disambig_id
    and output label diambig_id of that disambig symbol.

    Args:
      fst:
        It is changed in-place.
      start:
        The ID of #0
      end:
        The ID of the last disambig symbol. For instance if there are 3
        disambig symbols ``#0``, ``#1``, and ``#2``, then ``end`` is the ID
        of ``#2``.
    """
    for state in kaldifst.StateIterator(fst):
        for i in range(start, end + 1):
            fst.add_arc(
                state=state,
                arc=kaldifst.StdArc(
                    ilabel=i,
                    olabel=i,
                    weight=0,
                    nextstate=state,
                ),
            )

    if fst.output_symbols:
        for i in range(start, end + 1):
            fst.output_symbols.add_symbol(symbol=f"#{i-start}", key=i)



#!/usr/bin/env python3
#
# Copyright 2023 Xiaomi Corporation

import argparse
import logging
from pathlib import Path

import onnx
import torch
from model import RnnLmModel
from onnxruntime.quantization import QuantType, quantize_dynamic

from icefall.checkpoint import average_checkpoints, find_checkpoints, load_checkpoint
from icefall.utils import AttributeDict, str2bool
from typing import Dict
from train import get_params


def add_meta_data(filename: str, meta_data: Dict[str, str]):
    """Add meta data to an ONNX model. It is changed in-place.

    Args:
      filename:
        Filename of the ONNX model to be changed.
      meta_data:
        Key-value pairs.
    """
    model = onnx.load(filename)
    for key, value in meta_data.items():
        meta = model.metadata_props.add()
        meta.key = key
        meta.value = value

    onnx.save(model, filename)


# A wrapper for RnnLm model to simpily the C++ calling code
# when exporting the model to ONNX.
#
# TODO(fangjun): The current wrapper works only for non-streaming ASR
# since we don't expose the LM state and it is used to score
# a complete sentence at once.
class RnnLmModelWrapper(torch.nn.Module):
    def __init__(self, model: RnnLmModel, sos_id: int, eos_id: int):
        super().__init__()
        self.model = model
        self.sos_id = sos_id
        self.eos_id = eos_id

    def forward(self, x: torch.Tensor, x_lens: torch.Tensor) -> torch.Tensor:
        """
        Args:
          x:
            A 2-D tensor of shape (N, L) with dtype torch.int64.
            It does not contain SOS or EOS. We will add SOS and EOS inside
            this function.
          x_lens:
            A 1-D tensor of shape (N,) with dtype torch.int64. It contains
            number of valid tokens in ``x`` before padding.
        Returns:
          Return a 1-D tensor of shape (N,) containing negative loglikelihood.
          Its dtype is torch.float32
        """
        N = x.size(0)

        sos_tensor = torch.full((1,), fill_value=self.sos_id, dtype=x.dtype).expand(
            N, 1
        )
        sos_x = torch.cat([sos_tensor, x], dim=1)

        pad_col = torch.zeros((1,), dtype=x.dtype).expand(N, 1)
        x_eos = torch.cat([x, pad_col], dim=1)

        row_index = torch.arange(0, N, dtype=x.dtype)
        x_eos[row_index, x_lens] = self.eos_id

        # use x_lens + 1 here since we prepended x with sos
        return (
            self.model(x=sos_x, y=x_eos, lengths=x_lens + 1)
            .to(torch.float32)
            .sum(dim=1)
        )


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--epoch",
        type=int,
        default=29,
        help="It specifies the checkpoint to use for decoding."
        "Note: Epoch counts from 0.",
    )

    parser.add_argument(
        "--avg",
        type=int,
        default=5,
        help="Number of checkpoints to average. Automatically select "
        "consecutive checkpoints before the checkpoint specified by "
        "'--epoch'. ",
    )

    parser.add_argument(
        "--iter",
        type=int,
        default=0,
        help="""If positive, --epoch is ignored and it
        will use the checkpoint exp_dir/checkpoint-iter.pt.
        You can specify --avg to use more checkpoints for model averaging.
        """,
    )

    parser.add_argument(
        "--vocab-size",
        type=int,
        default=500,
        help="Vocabulary size of the model",
    )

    parser.add_argument(
        "--embedding-dim",
        type=int,
        default=2048,
        help="Embedding dim of the model",
    )

    parser.add_argument(
        "--hidden-dim",
        type=int,
        default=2048,
        help="Hidden dim of the model",
    )

    parser.add_argument(
        "--num-layers",
        type=int,
        default=3,
        help="Number of RNN layers the model",
    )

    parser.add_argument(
        "--tie-weights",
        type=str2bool,
        default=True,
        help="""True to share the weights between the input embedding layer and the
        last output linear layer
        """,
    )

    parser.add_argument(
        "--exp-dir",
        type=str,
        default="rnn_lm/exp",
        help="""It specifies the directory where all training related
        files, e.g., checkpoints, log, etc, are saved
        """,
    )

    return parser


def export_without_state(
    model: RnnLmModel,
    filename: str,
    params: AttributeDict,
    opset_version: int,
):
    model_wrapper = RnnLmModelWrapper(
        model,
        sos_id=params.sos_id,
        eos_id=params.eos_id,
    )

    N = 1
    L = 20
    x = torch.randint(low=1, high=params.vocab_size, size=(N, L), dtype=torch.int64)
    x_lens = torch.full((N,), fill_value=L, dtype=torch.int64)

    # Note(fangjun): The following warnings can be ignored.
    # We can use ./check-onnx.py to validate the exported model with batch_size > 1
    """
    torch/onnx/symbolic_opset9.py:2119: UserWarning: Exporting a model to ONNX
    with a batch_size other than 1, with a variable length with LSTM can cause
    an error when running the ONNX model with a different batch size. Make sure
    to save the model with a batch size of 1, or define the initial states
    (h0/c0) as inputs of the model. warnings.warn("Exporting a model to ONNX
    with a batch_size other than 1, " +
    """

    torch.onnx.export(
        model_wrapper,
        (x, x_lens),
        filename,
        verbose=False,
        opset_version=opset_version,
        input_names=["x", "x_lens"],
        output_names=["nll"],
        dynamic_axes={
            "x": {0: "N", 1: "L"},
            "x_lens": {0: "N"},
            "nll": {0: "N"},
        },
    )

    meta_data = {
        "model_type": "rnnlm",
        "version": "1",
        "model_author": "k2-fsa",
        "comment": "rnnlm without state",
        "sos_id": str(params.sos_id),
        "eos_id": str(params.eos_id),
        "vocab_size": str(params.vocab_size),
        "url": "https://huggingface.co/ezerhouni/icefall-librispeech-rnn-lm",
    }
    logging.info(f"meta_data: {meta_data}")

    add_meta_data(filename=filename, meta_data=meta_data)


def export_with_state(
    model: RnnLmModel,
    filename: str,
    params: AttributeDict,
    opset_version: int,
):
    N = 1
    L = 20
    num_layers = model.rnn.num_layers
    hidden_size = model.rnn.hidden_size
    embedding_dim = model.embedding_dim

    x = torch.randint(low=1, high=params.vocab_size, size=(N, L), dtype=torch.int64)
    h0 = torch.zeros(num_layers, N, hidden_size)
    c0 = torch.zeros(num_layers, N, hidden_size)

    # Note(fangjun): The following warnings can be ignored.
    # We can use ./check-onnx.py to validate the exported model with batch_size > 1
    """
    torch/onnx/symbolic_opset9.py:2119: UserWarning: Exporting a model to ONNX
    with a batch_size other than 1, with a variable length with LSTM can cause
    an error when running the ONNX model with a different batch size. Make sure
    to save the model with a batch size of 1, or define the initial states
    (h0/c0) as inputs of the model. warnings.warn("Exporting a model to ONNX
    with a batch_size other than 1, " +
    """

    torch.onnx.export(
        model,
        (x, h0, c0),
        filename,
        verbose=False,
        opset_version=opset_version,
        input_names=["x", "h0", "c0"],
        output_names=["log_softmax", "next_h0", "next_c0"],
        dynamic_axes={
            "x": {0: "N", 1: "L"},
            "h0": {1: "N"},
            "c0": {1: "N"},
            "log_softmax": {0: "N"},
            "next_h0": {1: "N"},
            "next_c0": {1: "N"},
        },
    )

    meta_data = {
        "model_type": "rnnlm",
        "version": "1",
        "model_author": "k2-fsa",
        "comment": "rnnlm state",
        "sos_id": str(params.sos_id),
        "eos_id": str(params.eos_id),
        "vocab_size": str(params.vocab_size),
        "num_layers": str(num_layers),
        "hidden_size": str(hidden_size),
        "embedding_dim": str(embedding_dim),
        "url": "https://huggingface.co/ezerhouni/icefall-librispeech-rnn-lm",
    }
    logging.info(f"meta_data: {meta_data}")

    add_meta_data(filename=filename, meta_data=meta_data)


@torch.no_grad()
def main():
    args = get_parser().parse_args()
    args.exp_dir = Path(args.exp_dir)

    params = get_params()
    params.update(vars(args))

    logging.info(params)

    device = torch.device("cpu")
    logging.info(f"device: {device}")

    model = RnnLmModel(
        vocab_size=params.vocab_size,
        embedding_dim=params.embedding_dim,
        hidden_dim=params.hidden_dim,
        num_layers=params.num_layers,
        tie_weights=params.tie_weights,
    )

    model.to(device)

    if params.iter > 0:
        filenames = find_checkpoints(params.exp_dir, iteration=-params.iter)[
            : params.avg
        ]
        if len(filenames) == 0:
            raise ValueError(
                f"No checkpoints found for --iter {params.iter}, --avg {params.avg}"
            )
        elif len(filenames) < params.avg:
            raise ValueError(
                f"Not enough checkpoints ({len(filenames)}) found for"
                f" --iter {params.iter}, --avg {params.avg}"
            )
        logging.info(f"averaging {filenames}")
        model.to(device)
        model.load_state_dict(
            average_checkpoints(filenames, device=device), strict=False
        )
    elif params.avg == 1:
        load_checkpoint(f"{params.exp_dir}/epoch-{params.epoch}.pt", model)
    else:
        start = params.epoch - params.avg + 1
        filenames = []
        for i in range(start, params.epoch + 1):
            if i >= 0:
                filenames.append(f"{params.exp_dir}/epoch-{i}.pt")
        logging.info(f"averaging {filenames}")
        model.to(device)
        model.load_state_dict(
            average_checkpoints(filenames, device=device), strict=False
        )

    model.to("cpu")
    model.eval()

    if params.iter > 0:
        suffix = f"iter-{params.iter}"
    else:
        suffix = f"epoch-{params.epoch}"

    suffix += f"-avg-{params.avg}"

    opset_version = 13

    logging.info("Exporting model without state")
    filename = params.exp_dir / f"no-state-{suffix}.onnx"
    export_without_state(
        model=model,
        filename=filename,
        params=params,
        opset_version=opset_version,
    )

    filename_int8 = params.exp_dir / f"no-state-{suffix}.int8.onnx"
    quantize_dynamic(
        model_input=filename,
        model_output=filename_int8,
        weight_type=QuantType.QInt8,
    )

    # now for streaming export
    saved_forward = model.__class__.forward
    model.__class__.forward = model.__class__.score_token_onnx
    streaming_filename = params.exp_dir / f"with-state-{suffix}.onnx"
    export_with_state(
        model=model,
        filename=streaming_filename,
        params=params,
        opset_version=opset_version,
    )
    model.__class__.forward = saved_forward

    streaming_filename_int8 = params.exp_dir / f"with-state-{suffix}.int8.onnx"
    quantize_dynamic(
        model_input=streaming_filename,
        model_output=streaming_filename_int8,
        weight_type=QuantType.QInt8,
    )


if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



# Please install OpenAI SDK first: `pip3 install openai`
import json
import random

import tqdm

from gxl_ai_utils.utils import utils_file

from openai import OpenAI

client = OpenAI(api_key="sk-d03313dd7c3a44a281187749c4203dda", base_url="https://api.deepseek.com")
print("Deepseek API client created successfully.")

prompt_person2person = '我希望构造$询问输入语音的说话人的情感信息$的句子，具体情况是一个人为另一个人播放了一段音频，并询问音频中说话人的情感。我将为你提供一些具体的限制，你来为我生成多样的句子。句式:{}。 情感一词的表述:"{}", 代词为"{}"（音频中的说话人的代词），音频的同义词为：" "{}",说话人的同义词："{}".请结合上述内容造对音频中说话人的情感的询问的句子。直接返回构造的句子，以json list的形式返回'
prompt_person2robot = '我希望构造$对机器人询问输入语音的说话人的情感信息$的句子，具体情况是一个人给LLM输入了一段音频，并询问音频中说话人的情感。我将为你提供一些具体的限制，你来为我生成多样的句子。句式:{}。 情感一词的表述:"{}",代词为"{}"（音频中的说话人的代词），音频的同义词为："{}",说话人的同义词："{}"， 输入的同义词为："{}".结合上述内容造对音频中说话人的情感的询问的句子。直接返回构造的句子，以json list的形式返回'

with open("meta_data/sentence.json", "r", encoding="utf-8") as file:
    sentence_list = json.loads(file.read())
    print(f'len: {len(sentence_list)}, {sentence_list}')
with open("meta_data/emotion_list.json", "r", encoding="utf-8") as file:
    emotion_list = json.loads(file.read())
    print(f'len: {len(emotion_list)}, {emotion_list}')
with open("meta_data/person.json", "r", encoding="utf-8") as file:
    person_list = json.loads(file.read())
    print(f'len: {len(person_list)}, {person_list}')
with open("meta_data/wav.json", "r", encoding="utf-8") as file:
    wav_list = json.loads(file.read())
    print(f'len: {len(wav_list)}, {wav_list}')
with open("meta_data/talker.json", "r", encoding="utf-8") as file:
    talk_list = json.loads(file.read())
    print(f'len: {len(talk_list)}, {talk_list}')
with open("meta_data/input_word.json", "r", encoding="utf-8") as file:
    input_word_list = json.loads(file.read())
    print(f'len: {len(input_word_list)}, {input_word_list}')

prompt_num = 0
for sentence_i in sentence_list:
    for emotion_i in emotion_list:
        for person_i in person_list:
            for i in range(6):
                wav_i = random.choice(wav_list)
                talk_i = random.choice(talk_list)
                prompt_num += 1
                prompt = prompt_person2person.format(sentence_i, emotion_i, person_i, wav_i, talk_i)
print(f"Total prompt num: {prompt_num}")
tqdm_obj = tqdm.tqdm(total=prompt_num)
for sentence_i in sentence_list:
    for emotion_i in emotion_list:
        for person_i in person_list:
            for i in range(6):
                wav_i = random.choice(wav_list)
                talk_i = random.choice(talk_list)
                prompt_num += 1
                tqdm_obj.update(1)
                prompt = prompt_person2person.format(sentence_i, emotion_i, person_i, wav_i, talk_i)
                response = client.chat.completions.create(
                    model="deepseek-chat",
                    temperature=1.5,
                    messages=[
                        {"role": "system", "content": "You are a helpful assistant"},
                        {"role": "user", "content": f"{prompt}"},
                    ],
                    stream=False
                )
                res = response.choices[0].message.content
                start_index = res.find("[")
                end_index = res.find("]") + 1
                res = res[start_index:end_index]
                str_list = json.loads(res)
                res_file = './output_data/res_emotion_p2p.txt'
                utils_file.makedir_for_file(res_file)
                with open(res_file, 'a', encoding='utf8') as f:
                    for s in str_list:
                        f.write(s + '\n')
tqdm_obj2 = tqdm.tqdm(total=prompt_num)
for sentence_i in sentence_list:
    for emotion_i in emotion_list:
        for person_i in person_list:
            for i in range(6):
                wav_i = random.choice(wav_list)
                talk_i = random.choice(talk_list)
                input_word_i = random.choice(input_word_list)
                prompt_num += 1
                tqdm_obj2.update(1)
                prompt = prompt_person2robot.format(sentence_i, emotion_i, person_i, wav_i, talk_i, input_word_i)
                response = client.chat.completions.create(
                    model="deepseek-chat",
                    temperature=1.3,
                    messages=[
                        {"role": "system", "content": "You are a helpful assistant"},
                        {"role": "user", "content": f"{prompt}"},
                    ],
                    stream=False
                )
                res = response.choices[0].message.content
                start_index = res.find("[")
                end_index = res.find("]") + 1
                res = res[start_index:end_index]
                str_list = json.loads(res)
                res_file = './output_data/res_emotion_p2r.txt'
                utils_file.makedir_for_file(res_file)
                with open(res_file, 'a', encoding='utf8') as f:
                    for s in str_list:
                        f.write(s + '\n')





# response = client.chat.completions.create(
#     model="deepseek-chat",
#     temperature=1.5,
#     messages=[
#         {"role": "system", "content": "You are a helpful assistant"},
#         {"role": "user", "content": f"{prompt}"},
#     ],
#     stream=False
# )
# res = response.choices[0].message.content
# print(res)
# start_index = res.find("[")
# end_index = res.find("]")+1
# res = res[start_index:end_index]
# str_list = json.loads(res)
# print(str_list)
# res_file = 'res.txt'
# with open(res_file, 'a', encoding='utf8') as f:
#     for s in str_list:
#         f.write(s+'\n')